/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "AdcDeviceProxy.h"

#include <QHostAddress>

#include "DominoDevice.h"

Q_DECLARE_METATYPE(QHostAddress)

AdcDeviceProxy::AdcDeviceProxy(DominoDevice *device, QObject *parent)
    : MstreamMregDevGlobal(device, pendingStatus, parent)
{
    qRegisterMetaType<QHostAddress>();
    connect(this, &AdcDeviceProxy::proxySetHostAddress,
            device, &DominoDevice::setHostAddress,
            Qt::QueuedConnection); // TODO: remove

    connect(device, &DominoDevice::statusUpdated,
            this, &AdcDeviceProxy::deviceStatusUpdated);
    setCreateDataDecoder(true);
}

// copied from TqdcDeviceGlob
void AdcDeviceProxy::deviceStatusUpdated(const AdcStatus &status)
{
    Q_ASSERT(getDeviceIndex() == status.getDeviceIndex());
    if (getDeviceIndex() != status.getDeviceIndex())
        return;

    pendingStatus = status;
}

void AdcDeviceProxy::setHostAddress(const DeviceIndex &index, const QHostAddress &address, int ttl)
{
    emit proxySetHostAddress(index, address, ttl);
}
