#include "CruMainWindow.h"
#include "FeLinkTree/FeLinkTreeModel.h"
#include "ui_CruMainWindow.h"
#include "CruSignalRouter.h"

#include "base/Globals.h"
#include "configuration-manager/ConfigurationManager.h"
#include "FeLinkDialog/FeLinkStatusDialog.h"
#include "ModularDeviceStatus.h"

CruMainWindow::CruMainWindow(QWidget *parent)
    : ReadoutAppMainWindow(parent)
    , ui(new Ui::CruMainWindow)
{
    ui->setupUi(this);
    setupMenu(ui->menuFile, ui->menuHelp, ui->menuOptions, ui->menuTools);
    configuration_manager->setDefaultConfig(BaseDeviceAppConfig());
    updateControlState();
    if(auto feLinkTreeModel = feLinkStatusDialog->getModel()) {
        ui->feLinkTreeView->setModel(feLinkTreeModel);

        connect(ui->actionClear_stats, &QAction::triggered,
                feLinkTreeModel, &FeLinkTreeModel::clearStats);
        connect(feLinkTreeModel, &QAbstractItemModel::modelReset,
                this, &CruMainWindow::modelReseted);
    }

    auto &signalRouter = CruSignalRouter::instance();
    signalRouter.setMainWindow(this);
    updateRunLogicVisiblity();

    connect(ui->actionPrint_FSM_inputs, &QAction::triggered,
            this, &CruMainWindow::printFsmInputs);
    connect(ui->pushButtonStart, &QPushButton::clicked, this, &CruMainWindow::performLocalStart);
    connect(ui->pushButtonStop, &QPushButton::clicked, this, &CruMainWindow::performLocalStop);

    connect(this, &ReadoutAppMainWindow::showRunLogicChanged,
            this, &CruMainWindow::setShowRunLogic);
}

CruMainWindow::~CruMainWindow()
{
    delete ui;
}

DeviceStatusTableViewFancy *CruMainWindow::getDeviceStatusTableView()
{
    return ui->deviceStatusTableView;
}

RunLogicWidget *CruMainWindow::getRunLogicWidget() const
{
    return ui->runLogicWidget;
}

TrigCtrlStatWidget *CruMainWindow::getTrigCtrlStatWidget() const
{
    return ui->trigCtrlStatWidget;
}

void CruMainWindow::updateControlState()
{
    bool inStateErrReady = curFsmState == FsmState::Ready || curFsmState == FsmState::Error;

   ui->pushButtonStart->setEnabled(curFsmState == FsmState::Ready && !underRemoteCtrl);
   ui->pushButtonStop->setEnabled(curFsmState == FsmState::Run && !underRemoteCtrl);

    if(auto feLinkTreeModel = feLinkStatusDialog->getModel()) {
        feLinkTreeModel->setEnabled(inStateErrReady || Globals::getDebugMode());
    }

//    controlStateEnable((inStateErrReady || Globals::getDebugMode())
//                       && curFsmState != FsmState::Calibrate);

//    ui->actionReset_MStream->setEnabled(inStateErrReady);
//    ui->actionSend_soft_reset->setEnabled(inStateErrReady);
//    ui->deviceStatusTableView->setControlEnabled(inStateErrReady);
}

void CruMainWindow::setConfig(const ConfigKey &key, const BaseDeviceAppConfig &_config)
{
    auto runLogicOn = _config.defaultSetup.runLogic.isAnyLimit();
    bool runLogicUpdated = runLogicOn != config.defaultSetup.runLogic.isAnyLimit();

    ReadoutAppMainWindow::setConfig(key, _config);

    (config.isRun() ? ui->radioButtonOpen
                    : ui->radioButtonClose)->setChecked(true);

    if(runLogicUpdated)
        updateRunLogicVisiblity();
}

void CruMainWindow::setShowRunLogic(bool show)
{
    if(showRunLogic == show)
        return;

    showRunLogic = show;
    updateRunLogicVisiblity();
}

void CruMainWindow::modelReseted()
{
    ui->feLinkTreeView->expandAll();
    if(auto m = ui->feLinkTreeView->model()) {
        for(auto c=0; c<m->columnCount(); ++c)
            ui->feLinkTreeView->resizeColumnToContents(c);
    }
}

void CruMainWindow::updateRunLogicVisiblity() const
{
    ui->groupBoxRunLogic->setVisible(showRunLogic ||
                                     config.defaultSetup.runLogic.isAnyLimit());
}

void CruMainWindow::on_radioButtonOpen_toggled(bool open)
{
    if(config.isRun() == open)
        return;
    config.setRunState(open);
    emit configChanged(config);
}

