#ifndef ETHFLASHDEV_H
#define ETHFLASHDEV_H

#include "ModularDeviceStatus.h"
#include "DfpFileTypes.h"
#include "DfpFsmDev.h"
#include "ModularMregDevice.h"
#include "base/DeviceIndex.h"
#include "base/fw_version.h"
#include "flash-prog/FlashDevTask.h"
#include "flash-prog/FpgaBitFile.h"

class DeviceIndex;
class FlashProgModule;

class EthFlashDev : public ModularMregDevice
{
    Q_OBJECT
public:
    EthFlashDev(const DeviceIndex &index, QObject *parent = nullptr);

signals:
    void devConnected(const DeviceIndex &, QString, const fw_version_t &) const;
    void fsmStateChanged(const DeviceIndex &, const FsmState &cur, const FsmState &old) const;
    void devFileChanged(const DeviceIndex &index, const QString &fileName) const;
    void printFsmInput() const;
    void imageWritten(const DeviceIndex &index, FlashImageType imageType, const QString &path) const;

public slots:
    bool devFileUpdated(const DeviceIndex &i, const QString &path);
    void autoDetectFile(const DeviceIndex &i, const QString &path);
    void startDevWrite(FlashImageType imageType);
    void startDevVerify(FlashImageType imageType);
    void startDevErase(FlashImageType imageType);
    void startTask(DFP_TASK task, FlashImageType imageType);
    void allowFsmTransit();
    void setSingleCheckMode(bool v);

private slots:
    void on_deviceConnected() override;
    void on_deviceDisconnected() override;
    void onFsmStateChange(const FsmState &state);
    void runFlashProgTasks();
    void onTaskFinished(const FlashDevTask &task);

protected:
    bool readStatus() override;

private:
    QSharedPointer<FlashProgModule> flashProgModule();
    bool readFile();
    DFP_FILE readFileInt();
    QString getProductSignature() const;

    ModularDeviceStatus status;

    DfpFsmDev fsm;
    FlashImageType taskImageType = FlashImageType::Regular;

    QString filePath;
    FpgaBitFile fpgaBitFile;
    QTimer * const taskTimer;
};

#endif // ETHFLASHDEV_H
