#ifndef FILEWATCHERCONFIGDISPATCHER_H
#define FILEWATCHERCONFIGDISPATCHER_H

#include "base/AbstractDispatcher.h"

class FileWatcherConfigDispatcherPrivate;
class ConfigIndexName;
struct FileWatcherConfig;

class FileWatcherConfigDispatcher : public AbstractDispatcher
{
    Q_OBJECT
    Q_DISABLE_COPY_MOVE(FileWatcherConfigDispatcher)
    explicit FileWatcherConfigDispatcher(QObject *parent = nullptr);

public:
    // Singleton
    static FileWatcherConfigDispatcher & instance()
    {
        // thread-safe since C++11
        static FileWatcherConfigDispatcher obj;
        return obj;
    }

    virtual ~FileWatcherConfigDispatcher() override;

    virtual void sendCache();

    template<class T>
    void subscribeConfig(T *obj) {
        connect(obj, &T::configChanged,
                this, &FileWatcherConfigDispatcher::setConfig);
        connect(this, &FileWatcherConfigDispatcher::configChanged,
                obj, &T::setConfig);
        logConnection(obj, "AppConfig");
        sendCache();
    }

    template<class T>
    void subscribeCS(T *obj) {
            connect(this, &FileWatcherConfigDispatcher::switch_to_config,
                    obj, &T::configSelectorChanged,
                    Qt::UniqueConnection);

            connect(obj, &T::switch_to_config,
                    this, &FileWatcherConfigDispatcher::configSelectorChanged,
                    Qt::UniqueConnection);
            logConnection(obj, "ConfigIndexName");
    }

    template<class T>
    void subscribeAllowRemoteControl(T *obj) {
        connect(obj, &T::allowRemoteControlChanged,
                this, &FileWatcherConfigDispatcher::setAllowRemoteControl,
                Qt::UniqueConnection);
        connect(this, &FileWatcherConfigDispatcher::allowRemoteControlChanged,
                obj, &T::setAllowRemoteControl,
                Qt::UniqueConnection);
        logConnection(obj, "AllowRemoteControl");
    }

signals:
    void switch_to_config(const ConfigIndexName &conf) const;
    void allowRemoteControlChanged(bool allow) const;
    void configChanged(const FileWatcherConfig &) const;

public slots:
    void configSelectorChanged(const ConfigIndexName &cin);
    virtual void setAllowRemoteControl(bool allow);
    virtual void setConfig(const FileWatcherConfig &newConfig);

private:
    Q_DECLARE_PRIVATE(FileWatcherConfigDispatcher)
    FileWatcherConfigDispatcherPrivate * const d_ptr;
};

#endif // FILEWATCHERCONFIGDISPATCHER_H
