#ifndef TCPCLIENT_H
#define TCPCLIENT_H

#include <QHostAddress>
#include <QMetaType>
#include <QTimer>
class QTcpSocket;
class QSemaphore;

Q_DECLARE_METATYPE(QHostAddress)

class TcpClient : public QObject
{
    Q_OBJECT
public:
    TcpClient(QSemaphore *freeSizeSem, QObject *parent=nullptr);
    ~TcpClient() override;

    void startReadOut();
    void stopReadOut();
    QString getHostAddr() { return hostAddr.toString(); }
    uint getHostPort() { return hostPort; }

public slots:
    void setAddress(const QHostAddress &, uint port);

private slots:
    void requestNewData();
    void readData();
    void displayError(QAbstractSocket::SocketError socketError);
    void connected();
    void disconnected();

signals:
    void dataAccuired(QByteArray);
    void connectedStateChanged(bool state);

private:

    QSemaphore *queueFreeSizeSem;
    bool enabled;
    QTcpSocket *tcpSocket;
    QHostAddress hostAddr;
    uint hostPort;
    QTimer reconnectTimer;
    QTimer delayReadTimer;
    int connectingAttempts;
    quint64 counter;
};

#endif // TCPCLIENT_H
