#include "HrbSpinBoxDelegate.h"

#include <QDoubleSpinBox>
#include <QDebug>
#include <QTimer>

HrbSpinBoxDelegate::HrbSpinBoxDelegate(QObject *parent)
    : QItemDelegate(parent)
    , commitDataTimer(new QTimer(this))
{
    commitDataTimer->setInterval(10);
    commitDataTimer->setSingleShot(true);
    connect(commitDataTimer, &QTimer::timeout,
            this, &HrbSpinBoxDelegate::flushCommitData);
}

QWidget *HrbSpinBoxDelegate::createEditor(QWidget *parent,
                                       const QStyleOptionViewItem &/* option */,
                                       const QModelIndex &index) const
{
     auto spin = new QDoubleSpinBox(parent);

     initSpinBox(spin);
//     double val = index.model()->data(index, Qt::EditRole).toDouble();
//     spin->setValue(val);
     connect(spin, &QDoubleSpinBox::textChanged,
             this, &HrbSpinBoxDelegate::dataChanged);

     return spin;
}

void HrbSpinBoxDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
{
    auto value = index.model()->data(index, Qt::EditRole).toDouble();
    auto spinBox = static_cast<QDoubleSpinBox*>(editor);
    if(spinBox->value() == value)
        return;
    spinBox->blockSignals(true);
    spinBox->setValue(value);
    spinBox->blockSignals(false);
}

void HrbSpinBoxDelegate::setModelData(QWidget *editor,
                                   QAbstractItemModel *model,
                                   const QModelIndex &index) const
{
    auto spin_box = static_cast<QDoubleSpinBox*>(editor);
    spin_box->interpretText();
    auto value = spin_box->value();
    if(model->data(index, Qt::EditRole).toDouble() != value)
        model->setData(index, value, Qt::EditRole);
}

void HrbSpinBoxDelegate::updateEditorGeometry(QWidget *editor,
                                           const QStyleOptionViewItem &option,
                                           const QModelIndex &/* index */) const
{
    editor->setGeometry(option.rect);
}

void HrbSpinBoxDelegate::initSpinBox(QDoubleSpinBox *sbAsd)
{
    sbAsd->setSuffix(" V");
    sbAsd->setSingleStep(0.005);
    sbAsd->setDecimals(3);
    sbAsd->setMaximum(2.5);
    sbAsd->setMinimum(-2.5);
//    sbAsd->clearFocus();
}

void HrbSpinBoxDelegate::dataChanged()
{
    if(commitDataWidget) {
        if(commitDataWidget == sender())
            return;
        flushCommitData();
    }
    commitDataWidget = sender();
    commitDataTimer->start();
}

void HrbSpinBoxDelegate::flushCommitData()
{
    if(!commitDataWidget)
        return;
    commitData(qobject_cast<QWidget*>(commitDataWidget));
    commitDataWidget = nullptr;
}

DeviceChModelItem *HrbSpinBoxDelegate::getItemByIndex(const QModelIndex &index) const
{
    return static_cast<DeviceChModelItem*>(index.internalPointer());
}
