#include "EvBConfigConverter.h"

#include "base/CoreTypeConverter.h"
#include "base/ConfigConverterUtil.h"
#include "EvBConfig.h"

namespace {
//struct ClientConfig;
constexpr const char *k_client_index = "client_index";
constexpr const char *k_enabled = "enabled";
constexpr const char *k_bufSize = "bufSize";

//struct ClientConfig;
constexpr const char *k_evNumEn = "evNumEn";
constexpr const char *k_evNumFileSplit = "evNumFileSplit";
constexpr const char *k_fileSizeEn = "fileSizeEn";
constexpr const char *k_fileSizeSplit = "fileSizeSplit";
constexpr const char *k_timeEn = "timeEn";
constexpr const char *k_timeFileSplit = "timeFileSplit";

//class EvBConfig;
constexpr const char *k_infOutputEn = "infOutputEn";
constexpr const char *k_combinerOut = "combinerOut";
constexpr const char *k_isRemoteEnabled = "isRemoteEnabled";
constexpr const char *k_advancedCounters = "advancedCounters";
constexpr const char *k_combinerOutTcpNum = "combinerOutTcpNum";
constexpr const char *k_startEvNumber = "startEvNumber";
constexpr const char *k_evNumberStep = "evNumberStep";
constexpr const char *k_storageDir = "storageDir";
constexpr const char *k_filePrefix = "filePrefix";
constexpr const char *k_addRunIndex = "addRunIndex";
constexpr const char *k_addRunNumber = "addRunNumber";
constexpr const char *k_addLocalProgramIndex = "addLocalProgramIndex";
constexpr const char *k_addTimestamp = "addTimestamp";
constexpr const char *k_protectFile = "protectFile";
constexpr const char *k_timeDistrEn = "timeDistrEn";
constexpr const char *k_outputBufSize = "outputBufSize";
constexpr const char *k_monitorPort = "monitorPort";
constexpr const char *k_fixTlvFormat = "fixTlvFormat";
constexpr const char *k_fileSplit = "fileSplit";
constexpr const char *k_allowBufferAutoResize = "allowBufferAutoResize";
constexpr const char *k_clients = "clients";

template<typename T>
void test_get_data(const QJsonObject &m, const QString& key, T *val) {
    auto it = m.find(key);
    if(it != m.end()) {
        *val = QVariant::fromValue(it.value()).value<T>();
    }
}
template<>
void test_get_data(const QJsonObject &m, const QString& key, quint64 *val) {
    auto it = m.find(key);
    if(it != m.end()) {
        *val = jsonValueToUInt64(it.value());
    }
}
template<>
void test_get_data(const QJsonObject &m, const QString& key, EvBClientMap *val) {
    auto it = m.find(key);
    if(it != m.end()) {
        *val = ConfigConverterUtil::toMap<ClientIndex, EvBClientConfig>(it.value());
    }
}
}

// ===================== ClientConfig Start =====================

template<>
bool AbstractConfigConverter<EvBClientConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<EvBClientConfig>::toJsonObject(const EvBClientConfig &in)
{
    QVariantMap m;
    m[k_client_index] = QVariant::fromValue(in.index);
    m[k_enabled] = QVariant::fromValue(in.enabled);
    m[k_bufSize] = jsonValueFromUInt64(in.bufSize);
    return fromVariantMap(m);
}

template<>
EvBClientConfig AbstractConfigConverter<EvBClientConfig>::fromJsonObject(const QJsonObject &m)
{
    EvBClientConfig out;
    update_value(m, k_client_index, &out.index);
    update_value(m, k_enabled, &out.enabled);
    out.bufSize = jsonValueToUInt64(m[k_bufSize]);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<EvBClientConfig>::toJsonValue(const EvBClientConfig &in)
{
    return toJsonObject(in);
}

template<>
EvBClientConfig AbstractConfigConverter<EvBClientConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<EvBClientConfig>::registerSubTypes()
{
    CoreTypeConverter::registerTypes();
    AbstractConfigConverter<ClientIndex>::registerTypes();
}

// ===================== ClientConfig End =====================

// ===================== EvbFileSplitterConfig Start =====================

template<>
bool AbstractConfigConverter<EvbFileSplitterConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<EvbFileSplitterConfig>::toJsonObject(const EvbFileSplitterConfig &in)
{
    QVariantMap m;
    m[k_evNumEn] = QVariant::fromValue(in.evNumEn);
    m[k_evNumFileSplit] = QVariant::fromValue(in.evNumFileSplit);
    m[k_fileSizeEn] = QVariant::fromValue(in.fileSizeEn);
    m[k_fileSizeSplit] = jsonValueFromUInt64(in.fileSizeSplit);
    m[k_timeEn] = QVariant::fromValue(in.timeEn);
    m[k_timeFileSplit] = QVariant::fromValue(in.timeFileSplit);
    return fromVariantMap(m);
}

template<>
EvbFileSplitterConfig AbstractConfigConverter<EvbFileSplitterConfig>::fromJsonObject(const QJsonObject &m)
{
    EvbFileSplitterConfig out;
    update_value(m, k_evNumEn, &out.evNumEn);
    update_value(m, k_evNumFileSplit, &out.evNumFileSplit);
    update_value(m, k_fileSizeEn, &out.fileSizeEn);
    update_value(m, k_fileSizeSplit, &out.fileSizeSplit);
    out.fileSizeSplit = jsonValueToUInt64(m[k_fileSizeSplit]);
    update_value(m, k_timeEn, &out.timeEn);
    update_value(m, k_timeFileSplit, &out.timeFileSplit);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<EvbFileSplitterConfig>::toJsonValue(const EvbFileSplitterConfig &in)
{
    return toJsonObject(in);
}

template<>
EvbFileSplitterConfig AbstractConfigConverter<EvbFileSplitterConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<EvbFileSplitterConfig>::registerSubTypes()
{
}

// ===================== EvbFileSplitterConfig End =====================

// ===================== EvBConfig Start =====================

template<>
bool AbstractConfigConverter<EvBConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<EvBConfig>::toJsonObject(const EvBConfig &in)
{
    QVariantMap m;
    m[k_combinerOut] = QVariant::fromValue(static_cast<int>(in.combinerCfg.combinerOut));
    m[k_combinerOutTcpNum] = QVariant::fromValue(in.combinerCfg.combinerOutTcpNum);
    m[k_outputBufSize] = jsonValueFromUInt64(in.combinerCfg.outputBufSize);
    m[k_infOutputEn] = QVariant::fromValue(in.combinerCfg.infOutputEn);
    m[k_timeDistrEn] = QVariant::fromValue(in.combinerCfg.timeDistrEn);
    m[k_fixTlvFormat] = QVariant::fromValue(in.combinerCfg.fixTlvFormat);

    m[k_storageDir] = QVariant::fromValue(in.writerCfg.storageDir);
    m[k_filePrefix] = QVariant::fromValue(in.writerCfg.filePrefix);
    m[k_addRunIndex] = QVariant::fromValue(in.writerCfg.addRunIndex);
    m[k_addRunNumber] = QVariant::fromValue(in.writerCfg.addRunNumber);
    m[k_addLocalProgramIndex] = QVariant::fromValue(in.writerCfg.addLocalProgramIndex);
    m[k_addTimestamp] = QVariant::fromValue(in.writerCfg.addTimestamp);
    m[k_protectFile] = QVariant::fromValue(in.writerCfg.protectFile);

    m[k_isRemoteEnabled] = QVariant::fromValue(in.isRemoteEnabled);
    m[k_advancedCounters] = QVariant::fromValue(in.advancedCounters);
    m[k_startEvNumber] = QVariant::fromValue(in.evNumCfg.startEvNum);
    m[k_evNumberStep] = QVariant::fromValue(in.evNumCfg.step);
    m[k_monitorPort] = QVariant::fromValue(in.monitorPort);
    m[k_fileSplit] = QVariant::fromValue(in.fileSplit);
    m[k_allowBufferAutoResize] = QVariant::fromValue(in.allowBufferAutoResize);
    m[k_clients] = ConfigConverterUtil::toJsonValue<ClientIndex, EvBClientConfig>(in.clients);
    return fromVariantMap(m);
}

template<>
EvBConfig AbstractConfigConverter<EvBConfig>::fromJsonObject(const QJsonObject &m)
{
    EvBConfig out;

    update_value(m, k_combinerOut, &out.combinerCfg.combinerOut);
    update_value(m, k_combinerOutTcpNum, &out.combinerCfg.combinerOutTcpNum);
    update_value(m, k_outputBufSize, &out.combinerCfg.outputBufSize);
    out.combinerCfg.outputBufSize = jsonValueToUInt64(m[k_outputBufSize]);
    update_value(m, k_infOutputEn, &out.combinerCfg.infOutputEn);
    update_value(m, k_timeDistrEn, &out.combinerCfg.timeDistrEn);
    update_value(m, k_fixTlvFormat, &out.combinerCfg.fixTlvFormat);

    update_value(m, k_storageDir, &out.writerCfg.storageDir);
    update_value(m, k_filePrefix, &out.writerCfg.filePrefix);
    update_value(m, k_addRunIndex, &out.writerCfg.addRunIndex);
    update_value(m, k_addRunNumber, &out.writerCfg.addRunNumber);
    update_value(m, k_addLocalProgramIndex, &out.writerCfg.addLocalProgramIndex);
    update_value(m, k_addTimestamp, &out.writerCfg.addTimestamp);
    update_value(m, k_protectFile, &out.writerCfg.protectFile);

    update_value(m, k_isRemoteEnabled, &out.isRemoteEnabled);
    update_value(m, k_advancedCounters, &out.advancedCounters);
    update_value(m, k_startEvNumber, &out.evNumCfg.startEvNum);
    update_value(m, k_evNumberStep, &out.evNumCfg.step);
    update_value(m, k_monitorPort, &out.monitorPort);
    update_value(m, k_fileSplit, &out.fileSplit);
    update_value(m, k_allowBufferAutoResize, &out.allowBufferAutoResize);
    out.clients = ConfigConverterUtil::toMap<ClientIndex, EvBClientConfig>
        (m[k_clients]);
    if(out.clients.remove(ClientIndex())) {
        qWarning() << "Some error in clients config";
    }
    return out;
}

template<>
QJsonValue AbstractConfigConverter<EvBConfig>::toJsonValue(const EvBConfig &in)
{
    return toJsonObject(in);
}

template<>
EvBConfig AbstractConfigConverter<EvBConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<EvBConfig>::registerSubTypes()
{
    CoreTypeConverter::registerTypes();

    AbstractConfigConverter<EvBClientConfig>::registerTypes();
    AbstractConfigConverter<EvbFileSplitterConfig>::registerTypes();
}
// ===================== EvBConfig End =====================
