#include "EvBConfigDispatcher.h"

#include "EvBConfig.h"

EvBConfigDispatcher::EvBConfigDispatcher(QObject *parent) :
    BaseConfigDispatcher(parent),
    config(new EvBConfig())
{}

void EvBConfigDispatcher::sendCache()
{
    BaseConfigDispatcher::sendCache();

    emit configChanged(*config);
    emit clientListChanged(config->getClientIndexEnabledMap());
    emit fileSplitterConfigChanged(config->fileSplit);
    for(auto cc : config->clients)
        emit clientConfigChanged(cc);
    emit combinerConfigChanged(config->combinerCfg);
    emit evNumberConfigChanged(config->evNumCfg);
    emit writerConfigChanged(config->writerCfg);
}

void EvBConfigDispatcher::setFileSplitterConfig(const EvbFileSplitterConfig &newConfig)
{
    if(config->fileSplit == newConfig)
        return;

    config->fileSplit = newConfig;
    emit configChanged(*config);
    emit fileSplitterConfigChanged(config->fileSplit);
}

void EvBConfigDispatcher::setConfig(const EvBConfig &newConfig)
{
    if (newConfig == *config)
        return;

    const auto &newClientsSet = newConfig.getClientIndexEnabledMap();
    if(newClientsSet != config->getClientIndexEnabledMap()){
        emit clientListChanged(newClientsSet);
    }

    *config = newConfig;
    emit configChanged(*config);
    emit fileSplitterConfigChanged(config->fileSplit);
    for(auto cc : config->clients)
        emit clientConfigChanged(cc);
    emit combinerConfigChanged(config->combinerCfg);
    emit evNumberConfigChanged(config->evNumCfg);
    emit writerConfigChanged(config->writerCfg);
}

void EvBConfigDispatcher::setClientList(const ClientIndexEnabledMap &new_clients)
{
    auto curClients = config->getClientIndexEnabledMap();
    ClientIndexEnabledMap filtered_clients(new_clients);
    filtered_clients.remove(ClientIndex()); // remove invalid index if present
    if (filtered_clients == curClients)
        return;

    const auto &oldSet = ClientIndexSet(curClients);
    const auto &newSet = ClientIndexSet(filtered_clients);
    curClients = filtered_clients;
    const auto toRemove = oldSet - newSet;
    for (const auto &index : toRemove)
        config->clients.remove(index);

    for (const auto &index : newSet) {
        auto &cfg = config->clients[index];
        cfg.index = index;
        cfg.enabled = new_clients[index];
        cfg.checkAndRepair();
    }

    sendCache();
    emit configChanged(*config);
    emit clientListChanged(curClients);
    for(auto cc : config->clients)
        emit clientConfigChanged(cc);
}

void EvBConfigDispatcher::setClientConfig(const EvBClientConfig &cfg)
{
    auto it = config->clients.find(cfg.index);
    if(it == config->clients.end())
        return;
    if(*it == cfg)
        return;
    *it = cfg;
    emit configChanged(*config);
    emit clientConfigChanged(*it);
}

void EvBConfigDispatcher::setCombinerConfig(const EvbCombinerConfig &cfg)
{
    if(config->combinerCfg == cfg)
        return;
    config->combinerCfg = cfg;
    emit configChanged(*config);
    emit combinerConfigChanged(config->combinerCfg);
}

void EvBConfigDispatcher::setEvbEvNumberConfig(const EvbEvNumberConfig &cfg)
{
    if(config->evNumCfg == cfg)
        return;
    config->evNumCfg = cfg;
    emit configChanged(*config);
    emit evNumberConfigChanged(config->evNumCfg);
}

void EvBConfigDispatcher::setWriterConfig(const EvbWriterConfig &cfg)
{
    if(config->writerCfg == cfg)
        return;
    config->writerCfg = cfg;
    emit configChanged(*config);
    emit writerConfigChanged(config->writerCfg);
}
