#include "FileSplitDialog.h"
#include "ui_FileSplitDialog.h"

#include "EvBConfig.h"

FileSplitDialog::FileSplitDialog(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::FileSplitDialog)
    , cfg(new EvbFileSplitterConfig())
{
    ui->setupUi(this);
    updateGui();
}

FileSplitDialog::~FileSplitDialog()
{
    delete ui;
    delete cfg;
}

void FileSplitDialog::setFileSplitterConfig(const EvbFileSplitterConfig &newConfig)
{
    if(*cfg == newConfig)
        return;
    *cfg = newConfig;
    updateGui();
}

void FileSplitDialog::on_groupBoxEvNum_toggled(bool en)
{
    if(cfg->evNumEn == en)
        return;
    cfg->evNumEn = en;
    emit fileSplitterConfigChanged(*cfg);
}

void FileSplitDialog::on_spinBoxEventNum_valueChanged(int val)
{
    if(cfg->evNumFileSplit == val)
        return;
    cfg->evNumFileSplit = val;
    emit fileSplitterConfigChanged(*cfg);
}

void FileSplitDialog::on_groupBoxFileSize_toggled(bool en)
{
    if(cfg->fileSizeEn == en)
        return;
    cfg->fileSizeEn = en;
    emit fileSplitterConfigChanged(*cfg);
}

void FileSplitDialog::on_spinBoxFileSize_valueChanged(int val)
{
    quint64 fileSize = quint64(val)*1e6;
    if(cfg->fileSizeSplit == fileSize)
        return;
    cfg->fileSizeSplit = fileSize;
    emit fileSplitterConfigChanged(*cfg);
}

void FileSplitDialog::on_groupBoxTime_toggled(bool en)
{
    if(cfg->timeEn == en)
        return;
    cfg->timeEn = en;
    emit fileSplitterConfigChanged(*cfg);
}

void FileSplitDialog::on_spinBoxTime_valueChanged(int val)
{
    if(cfg->timeFileSplit == val)
        return;
    cfg->timeFileSplit = val;
    emit fileSplitterConfigChanged(*cfg);
}

void FileSplitDialog::updateGui() const
{
    ui->groupBoxEvNum->setChecked(cfg->evNumEn);
    ui->spinBoxEventNum->setValue(cfg->evNumFileSplit);
    ui->groupBoxFileSize->setChecked(cfg->fileSizeEn);
    ui->spinBoxFileSize->setValue(cfg->fileSizeSplit/1000000);
    ui->groupBoxTime->setChecked(cfg->timeEn);
    ui->spinBoxTime->setValue(cfg->timeFileSplit);
}
