#ifndef STATISTICDIALOG_H
#define STATISTICDIALOG_H

#include <QDialog>
#include <QMap>

#include "Analizer.h"
#include "Combiner.h"
#include "FileWriterStats.h"
#include "TcpClient.h"
class QTableWidget;

namespace Ui {
class StatisticDialog;
}

class StatisticDialog : public QDialog
{
    Q_OBJECT
public:
    enum class Table {
        Tcp,
        Anal,
        CombClient,
        Writer,
        OutTcp
    };

    explicit StatisticDialog(QWidget *parent = nullptr);
    ~StatisticDialog();

public slots:
    virtual void setVisible(bool visible);
    void analizerListUpdated(const ClientIndexSet &clientIds);
    void analyzerStatUpdated(const AnalyzerStat &s);
    void updateTcpStatus(const TcpClientStatus &status);
    void updateCombinerStats(const CombinerStats &combStat);
    void statusUpdated(const FileWriterStats &s);

signals: void combinerConfigChanged(const EvbCombinerConfig &) const;
public slots: void setCombinerConfig(const EvbCombinerConfig &cfg);

private slots:
    void updateStat();

private:
    void init();
    void updateTableSize(QTableWidget *table);
    QTableWidget * getTableWidget(Table t) const;
    int getTableClientIndexCol(Table t) const;
    int getRowByClientIndex(Table t, const ClientIndex &i) const;
    ClientIndex getClientIndexByRow(Table t, int r) const;
    int getRowByIndex(Table t, int id) const;
    int getIndexByRow(Table t, int r) const;
    void updateAnalStat();
    void updateTcpStat();
    void updateCombClientsStats(const CombinerStats &combStat);
    void updateFWStats(const CombinerStats &combStat);
    void updateOutTcpStats(const CombinerStats &combStat);

    Ui::StatisticDialog *ui;
    ClientIndexSet curClientIds;
    QMap<ClientIndex, AnalyzerStat> analyzerStats;
    QMap<ClientIndex, TcpStatistic> tcpStats;
    std::optional<CombinerStats> pendingCombStat;
    CombinerStats prevCombStat;
    FileWriterStats prevFwStat;
    QTimer * const refreshTimer;
};

#endif // STATISTICDIALOG_H
