#ifndef TCPCLIENT_H
#define TCPCLIENT_H

#include <QElapsedTimer>
#include <QHostAddress>

#include "base/ClientIndex.h"

#include "TcpClientFsm.h"
#include "TcpClientStatus.h"

class DnsLookup;
template<class T> class RingBuffer;
struct EvBClientConfig;

class QTimer;

class TcpClient : public TcpClientFsm
{
    Q_OBJECT
public:
    TcpClient(ClientIndex i, int _evNumIndex, RingBuffer<char> *byteRB, quint64 inputBufSize);
    virtual ~TcpClient();

public slots:
    void setReadOutEn(bool en);
    void setAddress(const ProgramDescription &descr, const ProgramInterface &interface);
    void pnpCacheUpdated(const PnpInfoCache &cache);
    void setClientConfig(const EvBClientConfig &cgf);

private slots:
    void connectToHost();
    void readData();
    void doSendMetrics();
    void updateStatus();
    void reverseEntryUpdated(const QPair<quint32, QString> &ent);
    void resizeBuffer();
    void low_space_timer_slot();

signals:
    void dataAcquired(ClientIndex index, QByteArray);
    void sendStatus(const TcpClientStatus &status);
    void inputBufferSizeUpdated(const ClientIndex &index, quint64 size) const;
    void clientConfigChanged(const EvBClientConfig &) const;

protected:
    void doStateAction(const FsmState &st) final;

private:
    void disconnectFromHost();
    bool isConnectionAlive() const;
    void resetReadOut();
    bool connectToServer();
    bool receiveFromPeer();
    int updateContFreeSize();
    int updateFreeSize();
    void checkFreeSizeWaterMark();

    const ClientIndex index;
    int evNumIndex;
    bool readoutEnabled = true;
    std::optional<int> socketFd;

    DnsLookup * const dnsLookup;
    QHostAddress hostAddr;
    QString hostName;
    quint32 hostPort = 0;
    QTimer * const reconnectTimer;
    QTimer * const readTimer;
    QTimer * const statusTimer;
    QTimer * const sendMetricsTimer;
    QTimer * const resizeTimer;
    QTimer * const low_space_timer;
    int connectingAttempts = 0;
    quint64 packageCounter = 0;

    QByteArray idleBuf;
    RingBuffer<char> * const byteRB;
    bool noFreeSpace = false;
    QElapsedTimer lastNoFreeSpace;
    TcpClientStatus status;

    bool srcBusy = false;
    bool srcBusyWarn = true;
    QString suppresConnectionWarn;
    bool failToResize = false;
    int freeSizeCache=0;
    int contFreeSizeCache=0;
};

#endif // TCPCLIENT_H
