/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "Ad5328/Ad5328Module.h"
#include "MscDacCalibrator.h"
#include "MscDacConverter.h"
#include "MscInput/MscInputModule.h"
#include "device-utils/Ad5328ConfigBuilder.h"


bool MscDacCalibrator::writeLevels(const QMap<int, int> &values)
{
    if (!thresholdMod)
        return {};
    auto ad5328Config = dacConfigBuilder.build(values);
    return thresholdMod->writeConfig(ad5328Config);
}

bool MscDacCalibrator::readStates(AllChannelState *states)
{
    if (!hitCore)
        return {};
    const auto &chStates = hitCore->readCmpState(num_samples);
    if (chStates.empty())
        return {};
    if (states) {
        *states = calcProbabilities(chStates, getNumberOfChannels());
    }
    return true;
}

std::optional<DacCalResult> MscDacCalibrator::calibrateDac()
{
    const int cal_dac_delta_mv = 1000;
    const quint16 dacMax = MscDacConverter::instance().dacValue(cal_dac_delta_mv);
    const quint16 dacMin = MscDacConverter::instance().dacValue(-cal_dac_delta_mv);
    return  AbstractThresholdCalibrator::calibrateDac(dacMin, dacMax);
}

double MscDacCalibrator::convert_raw_to_mV(double raw) const
{
    return MscDacConverter::instance().dac_mV(raw);
}

int MscDacCalibrator::getNumberOfChannels() const
{
    return 16;
}
