/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "MscDacConfigBuilder.h"
#include "MscDacConverter.h"

namespace {

static const int CHIP_COUNT = 2;
static const int CHIP_CHANNELS = 8;

} // namespace

Ad5328ConfigBuilder::ChannelIndex MscDacConfigBuilder::mapChannel(int channel) const
{
    ChannelIndex channelIndex;
    if (channel < 0 || channel >= CHIP_COUNT * CHIP_CHANNELS)
        return {};
    channelIndex.chip = channel / CHIP_CHANNELS;
    channelIndex.chipChannel = CHIP_CHANNELS - 1 - (channel % CHIP_CHANNELS);
    return channelIndex;
}

Ad5328Config MscDacConfigBuilder::build(const QMap<int, int> &values) const
{
    Ad5328Config config;

    for (auto it = values.begin(); it != values.end(); ++it) {
        int ch = it.key();
        auto dacValue = it.value();
        auto index = mapChannel(ch);
        auto &chip = config.chips[index.chip];
        chip.doubleGain = MscDacConverter::instance().getGain();
        chip.dacValue[index.chipChannel] = dacValue;
    }
    return config;
}
