#ifndef MSCTHRESHOLDMODEL_H
#define MSCTHRESHOLDMODEL_H

#include <QAbstractItemModel>

#include "MscThresholdColumns.h"
#include "lib-common/BaseDeviceAppConfig.h"
#include "status/MscStatus.h"

class ConfigKey;
struct MscModelItem;

class MscThresholdModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    MscThresholdModel(QObject *parent = nullptr);
    QVariant data(const QModelIndex &index, int role) const override;
    virtual bool setData(const QModelIndex &model_index,
                         const QVariant &value,
                         int role = Qt::EditRole) override;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role) const override;
    QModelIndex index(int row, int column,
                      const QModelIndex &parent = QModelIndex()) const override;
    QModelIndex parent(const QModelIndex &index) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override
    {
        Q_UNUSED(parent)
        return static_cast<int>(MscThresholdColumns::COL_TOTAL);
    }
public slots:
    void statusUpdated(const MscStatus &status);

signals: void configChanged(const BaseDeviceAppConfig &config) const;
public slots: void setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig);

private:
    QVariant displayData(const QModelIndex &index) const;
    QVariant editData(const QModelIndex &index) const;
    QVariant foregroundRoleData(const QModelIndex &index) const;
    void columnDataChanged(MscThresholdColumns col);
    void reset();
    inline MscModelItem *getItemByIndex(const QModelIndex &index) const;
    inline MscThresholdColumns getColumnByIndex(const QModelIndex &index) const;

    MscModelItem * const rootItem;
    BaseDeviceAppConfig config;
    MscStatus devStatus;
    QMap<DeviceIndex, int> chNum;
};

#endif // MSCTHRESHOLDMODEL_H
