/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef CORETYPES_H
#define CORETYPES_H

#include <QList>
#include <QMap>
#include <QMetaType>
#include <QStringList>

#include "base/RcRunId.h"

enum class SlaveType { Unknown, Trig, VmeTrig, TcpClient };
enum class SlaveMessageSeverity { Warning, Error };

struct RcSlaveId
{
    RcSlaveId(SlaveType type = SlaveType::Unknown, const QString &slaveId = QString())
        : type(type), slaveId(slaveId) { }

    SlaveType type = SlaveType::Unknown;
    QString slaveId;
};

struct SeverityMessage
{
    SeverityMessage(SlaveMessageSeverity severity = SlaveMessageSeverity::Warning,
                    QString text = QString())
        : severity(severity), text(text) { }

    bool isEmpty() const { return text.isEmpty(); }

    SlaveMessageSeverity severity = SlaveMessageSeverity::Warning;
    QString text;
};

struct MessageTreeItem
{
    MessageTreeItem(RcSlaveId id = RcSlaveId(),
                   const RcRunId &runId = RcRunId(),
                   const SeverityMessage &sm = SeverityMessage())
        : id(id), runId(runId), sm(sm) { }

    bool isEmpty() const { return sm.isEmpty(); }
    SlaveType type() const { return id.type; }
    SlaveMessageSeverity severity() const { return sm.severity; }
    QString text() const { return sm.text; }

    RcSlaveId id;
    RcRunId runId;
    SeverityMessage sm;
};

Q_DECLARE_METATYPE(MessageTreeItem)

using RcSlaveMessageList = QList<MessageTreeItem>;

struct MessageTree
{
    MessageTreeItem m;
    QList<MessageTree> branches;
};

//struct RcSlaveStatus
//{
//    QMap<SlaveType, QMap<SlaveMessageSeverity, QStringList> > messages;
//};

#endif // CORETYPES_H
