#ifndef TQDCCONFIGDISPATCHER_H
#define TQDCCONFIGDISPATCHER_H

#include "lib-common/ReadoutAppConfigDispatcher.h"

class BaseDeviceAppConfig;
class CalConfig;
class CalResult;
class DeviceIndexEnabledMap;
class BaseDeviceAppConfig;
class TtlIoConfig;

class TqdcConfigDispatcher : public ReadoutAppConfigDispatcher
{
Q_OBJECT
    explicit TqdcConfigDispatcher(QObject *parent = nullptr);
    Q_DISABLE_COPY_MOVE(TqdcConfigDispatcher);
public:
    static TqdcConfigDispatcher & instance()
    {
        static TqdcConfigDispatcher obj;
        return obj;
    }

    template<class T>
    void subscribeCalConfig(T *obj) {
        connect(this, &TqdcConfigDispatcher::calConfigChanged,
                obj, &T::setCalConfig,
                Qt::UniqueConnection);

        connect(obj, &T::calConfigChanged,
                this, &TqdcConfigDispatcher::setCalConfig,
                Qt::UniqueConnection);
        logConnection(obj, "CalConfig");
    }

    template<class T>
    void subscribeCalResult(T *obj) {
        connect(this, &TqdcConfigDispatcher::calResultChanged,
                obj, &T::setCalResult,
                Qt::UniqueConnection);

        connect(obj, &T::calResultChanged,
                this, &TqdcConfigDispatcher::setCalResult,
                Qt::UniqueConnection);
        logConnection(obj, "CalResult");
    }

signals:
    void calConfigChanged(ConfigKey key, const CalConfig &) const;
    void calResultChanged(ConfigKey key, const CalResult &) const;

public slots:
    void setCalConfig(const CalConfig &newCalConfig);
    void setCalResult(const CalResult &newCalResult);

private:
    void sendConfigKeyCache() override;

    boost::shared_ptr<CalConfig> cacheCalConfig;
};
#endif // TQDCCONFIGDISPATCHER_H
