#include "TtvxsAuxCounters.h"

#include <QDateTime>

namespace {
const int TTVXS_CLOCK = 125000000;

const char AUX_CNT_DSCR2[28][40] = {
    "Uptime",
    "Resulting Busy Time",
    "LEMO Trigger Pulse Count",
    "SFP Trigger Pulse Count",
//    "Trigger Candidates Count",
//    "Accepted Trigger Count",
    "Channel 1 Busy Time",
    "Channel 2 Busy Time",
    "Channel 3 Busy Time",
    "Channel 4 Busy Time",
    "Channel 5 Busy Time",
    "Channel 6 Busy Time",
    "Channel 7 Busy Time",
    "Channel 8 Busy Time",
    "Channel 9 Busy Time",
    "Channel 10 Busy Time",
    "Channel 11 Busy Time",
    "Channel 12 Busy Time",
    "Channel 13 Busy Time",
    "Channel 14 Busy Time",
    "Channel 15 Busy Time",
    "Channel 16 Busy Time",
    "Channel 17 Busy Time",
    "Channel 18 Busy Time",
    "Resulting Busy Time",
    "Run Uptime",
    "LEMO Trigger Pulse Count",
    "SFP Trigger Pulse Count",
    "Trigger Candidates Count",
    "Accepted Trigger Count"};

QString clock_to_time_str(quint64 t) {
    quint64 msec = t * 1000;
    quint64 sec = msec/1000;
    quint64 min = sec/60;
    quint64 h = min/60;
    QString str = QString("%1:%2:%3")
            .arg(h)
            .arg(min % 60, 2, 10, QChar('0'))
            .arg(sec % 60, 2, 10, QChar('0'));
    return str;
}
}

TtvxsAuxCounters::TtvxsAuxCounters()
{
    data.resize(CNT_NUM);
}

QString TtvxsAuxCounters::getAuxDescr(int i)
{
    if(i<0||i>=TtvxsAuxCounters::CNT_NUM)
        return QString();
    return AUX_CNT_DSCR2[i];
}

QString TtvxsAuxCounters::getUpTime() const
{ return clock_to_time_str(getUpTimeSec());}

quint64 TtvxsAuxCounters::getUpTimeSec() const
{ return data[0]/TTVXS_CLOCK; }

QString TtvxsAuxCounters::getTtlCnt() const
{ return QString("%L1").arg(data[2]); }

QString TtvxsAuxCounters::getTtlRate(const TtvxsAuxCounters &reper) const
{
    const qint64 dt = data[0]-reper.data[0];
    if(dt) {
        qint64 rate = TTVXS_CLOCK*(data[2]-reper.data[2])/dt;
        return QString("%L1 Hz").arg(rate);
    } else
        return "-";
}

QString TtvxsAuxCounters::getSfpCnt() const
{ return QString("%L1").arg(data[3]); }

QString TtvxsAuxCounters::getSfpRate(const TtvxsAuxCounters &reper) const
{
    const qint64 dt = data[0]-reper.data[0];
    if(dt) {
        qint64 rate = TTVXS_CLOCK*(data[3]-reper.data[3])/dt;
        return QString("%L1 Hz").arg(rate);
    } else
        return "-";
}

QString TtvxsAuxCounters::getBusyTime(int ch) const
{ return data[27] ? QString("%1 us").arg(getBusyTimeUs(ch),0,'f',2) : ""; }

double TtvxsAuxCounters::getBusyTimeUs(int ch) const
{ return data[27] ? 1e6*data[ch+4]/data[27]/TTVXS_CLOCK : 0; }

QString TtvxsAuxCounters::getResultingBusyTime() const
{ return data[27] ? QString("%1 us").arg(getResultingBusyTimeUs(),0,'f',2) : ""; }

double TtvxsAuxCounters::getResultingBusyTimeUs() const
{ return data[27]? 1e6*data[22]/data[27]/TTVXS_CLOCK : 0;}

QString TtvxsAuxCounters::getRunTime() const
{ return clock_to_time_str(getRunTimeSec()); }

quint64 TtvxsAuxCounters::getRunTimeSec() const
{ return data[23]/TTVXS_CLOCK; }

QString TtvxsAuxCounters::getTtlRunCnt() const
{ return QString("%L1").arg(data[24]); }

QString TtvxsAuxCounters::getSfpRunCnt() const
{ return QString("%L1").arg(data[25]); }

QString TtvxsAuxCounters::getTrigRunCandCnt() const
{ return QString("%L1").arg(data[26]); }

QString TtvxsAuxCounters::getTrigRunAcceptCnt() const
{ return QString("%L1").arg(data[27]); }

QString TtvxsAuxCounters::getRunTrigRate(const TtvxsAuxCounters &reper) const
{
    const qint64 dt = data[23]-reper.data[23];
    if(dt) {
        qint64 rate = TTVXS_CLOCK*(data[27]-reper.data[27])/dt;
        return QString("%L1 Hz").arg(rate);
    } else
        return "-";
}
