#include "CalibrationDialog.h"
#include "ui_CalibrationDialog.h"

#include "AfiViewerConfig.h"
#include "TableHelper.h"

#include <QDebug>
#include <QTableWidget>

namespace {
enum {COL_CAL_CH, COL_CAL_INL, COL_CAL_TOTAL};
}

CalibrationDialog::CalibrationDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::CalibrationDialog),
    chNum(0)
{
    ui->setupUi(this);
    initTable();
}

CalibrationDialog::~CalibrationDialog()
{
    delete ui;
}

void CalibrationDialog::setChNumber(int num)
{
    if(chNum == num)
        return;
    QTableWidget *table = ui->tableWidget;
    chNum = num;
    table->setRowCount(chNum);
    QString txt;
    for(int ch=0; ch<chNum; ++ch) {
        txt = QString::number(ch+1);
        table->setItem(ch, COL_CAL_CH, new QTableWidgetItem(txt));

        table->setItem(ch, COL_CAL_INL, new QTableWidgetItem());
    }
    TableHelper::updateTableSize(ui->tableWidget);
}

void CalibrationDialog::init(const DeviceCalibration &devCal)
{
    setChNumber(devCal.nch);
    for(int ch=0; ch<chNum; ++ch) {
        inlPresent[ch] = false;
        setTableInl(ch, false);
    }
    updateInl(devCal.inl);
}

void CalibrationDialog::updateInl(const QMap<int, QVector<double> > &inlCorr)
{
    for (auto ch: inlCorr.keys()) {
        inlPresent[ch] = inlCorr.value(ch).size()==1024;
        setTableInl(ch, inlPresent[ch]);
    }
}

void CalibrationDialog::initTable()
{
    QTableWidget *table = ui->tableWidget;
    table->verticalHeader()->hide();
    table->setColumnCount(COL_CAL_TOTAL);
    table->setHorizontalHeaderItem(COL_CAL_CH, new QTableWidgetItem("Ch #"));
    table->setHorizontalHeaderItem(COL_CAL_INL, new QTableWidgetItem("INL corr."));

    table->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
    table->verticalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
//    updateTableSize(table);
//    setTableRowsCount(16);
}

void CalibrationDialog::setTableInl(int ch, bool inlOk)
{
    QTableWidgetItem *i = ui->tableWidget->item(ch, COL_CAL_INL);
    if(i){
        i->setText(inlOk ? "Yes":"No");
        auto brush = i->foreground();
        brush.setColor(inlOk ? Qt::darkGreen : Qt::red);
        i->setForeground(brush);
    } else {
        qCritical() << "There is no cell for inl cal.";
    }
}

void CalibrationDialog::on_pushButtonClose_clicked()
{
    close();
}
