#include "ViewerZmqPublisher.h"

#include <QDebug>
#include <QSemaphore>

#include "histogramm/DeviceData.h"

ViewerZmqPublisher::ViewerZmqPublisher(QSemaphore *_zmqSem, QObject *parent)
    : ZmqPublisher(parent)
    , zmqSem(_zmqSem)
{}

void ViewerZmqPublisher::publish_event(const DecodedData &data)
{
    zmqSem->release();
    if(!socket) return;
    if(!cfg.send_all) {
        if(zmqSem->available() <= 1)
            return;
    }

    for(auto totEvIt=data.events.cbegin(); totEvIt!=data.events.cend(); ++totEvIt) {
        for(auto devIt=totEvIt->tqdcEv.cbegin(); devIt != totEvIt->tqdcEv.cend(); ++devIt) {
            const auto &ev = devIt.value();
            if(ev.index.isNull())
                continue;

            QString mess_str = QString("%1 0x%2 Ev_%3")
                    .arg(ev.index.getDeviceIdName())
                    .arg(ev.index.getSerial(), 8, 16, QChar('0'))
                    .arg(ev.evNum);
            mess_str += QString("; time:%1.%2 %3valid")
                    .arg(ev.tai.sec)
                    .arg(ev.tai.ns)
                    .arg(ev.tai.valid?"":"in");

            for(auto chIt = ev.hits.begin(); chIt != ev.hits.end(); ++chIt) {
                for(const TdcChHit &tdc_ch_hit : chIt->tdcHits) {
                    mess_str += QString("; ch:%1").arg(tdc_ch_hit.ch);
                    if(tdc_ch_hit.leadingTime)
                        mess_str += QString(" l:%1").arg(tdc_ch_hit.leadingTime);
                    // TQSC16VS-E doesn't have trailing time
//                    if(tdc_ch_hit.trailingTime)
//                        mess_str += QString(" t:%1").arg(tdc_ch_hit.trailingTime);
                }

                for(const AdcChHit &adc_ch_hit : chIt->adcHits) {
                    mess_str += QString("; ch:%1 t:%2")
                            .arg(adc_ch_hit.ch).arg(adc_ch_hit.adcTs);
                    for (const double &sampe : adc_ch_hit.wf) {
                        mess_str += QString(" %1").arg(sampe, 0, 'f', 0);
                    }
                }
            }
            publish(mess_str);
        }
    }
}
