/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef CONFIGINDEXNAME_H
#define CONFIGINDEXNAME_H

#include <QMetaType>
#include <QString>

#include "ConfigurationName.h"
#include "ProgramIndex.h"

class ConfigIndexName {
public:
    ConfigIndexName(const ProgramIndex &index = ProgramIndex(),
                    const ConfigurationName &name = ConfigurationName())
        : program_index(index),
          configuration_name(name)
    {
        if (program_index.isEmpty())
            program_index = ProgramIndex();
        if (configuration_name.isEmpty())
            configuration_name = ConfigurationName();
    }

    friend bool operator==(const ConfigIndexName &l, const ConfigIndexName &r) {
        return l.program_index == r.program_index &&
                l.configuration_name == r.configuration_name;
    }
    friend bool operator!=(const ConfigIndexName &l, const ConfigIndexName &r) {
        return ! (l == r);
    }
    friend bool operator<(const ConfigIndexName &l, const ConfigIndexName &r) {
        if (l.program_index == r.program_index)
            return l.configuration_name < r.configuration_name;
        return l.program_index < r.program_index;
    }
    QString toString() const {
        return QString("%1/%2").arg(program_index.toString()).arg(configuration_name);
    }
    ProgramIndex getIndex() const {
        return program_index;
    }
    ConfigurationName getConfigurationName() const {
        return configuration_name;
    }

    ProgramIndex program_index;
    ConfigurationName configuration_name;
};

Q_DECLARE_METATYPE(ConfigIndexName)


inline QDataStream &operator<<(QDataStream &out, const ConfigIndexName &pod) {
    out<< pod.program_index<< pod.configuration_name;
    return out;
}

inline QDataStream &operator>>(QDataStream &in, ConfigIndexName &pod) {
    ProgramIndex program_index;
    ConfigurationName configuration_name;
    in >> program_index >> configuration_name;
    pod = ConfigIndexName(program_index, configuration_name);
    return in;
}

#endif // CONFIGINDEXNAME_H
