/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "DeviceIndexConverter.h"
#include <QMetaType>
namespace {
constexpr const char *k_device_id = "device_id";
constexpr const char *k_serial_id = "serial_id";
}

template<>
bool AbstractConfigConverter<DeviceIndex>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<DeviceIndex>::toJsonObject(const DeviceIndex &in)
{
    QVariantMap m;
    m[k_device_id] = QVariant::fromValue(static_cast<quint32>(in.getDevId()));
    m[k_serial_id] = QVariant::fromValue(in.getSerial());
    return fromVariantMap(m);
}

template<>
DeviceIndex AbstractConfigConverter<DeviceIndex>::fromJsonObject(const QJsonObject &m)
{
    auto device_id = static_cast<DeviceId>(QVariant::fromValue(m[k_device_id]).value<quint32>());
    auto serial_id = QVariant::fromValue(m[k_serial_id]).value<quint64>();
    return DeviceIndex(device_id, serial_id);
}

template<>
QJsonValue AbstractConfigConverter<DeviceIndex>::toJsonValue(const DeviceIndex &in)
{
    return toJsonObject(in);
}

template<>
DeviceIndex AbstractConfigConverter<DeviceIndex>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<DeviceIndex>::registerSubTypes()
{
    if (QMetaType::hasRegisteredConverterFunction<QString, DeviceIndex>())
        return;
#if (QT_VERSION < QT_VERSION_CHECK(6,0,0))
    qRegisterMetaTypeStreamOperators<DeviceIndex>();
    QMetaType::registerComparators<DeviceIndex>();
#endif

    QMetaType::registerConverter<DeviceIndex, QString>(&DeviceIndex::toString);
    QMetaType::registerConverter<QString, DeviceIndex>(&DeviceIndex::fromString);
}
