/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef STATCOUNTER_H
#define STATCOUNTER_H

#include <QDebug>
#include <QMap>
#include <QVector>

class StatCounter
{
public:
    static uint64_t update_with_rollover_fix(uint64_t wide, uint64_t newval, int width);
    static uint64_t update_with_rollover_fix(uint64_t wide, uint32_t newval);
    static uint64_t update_with_rollover_fix(uint64_t wide, uint16_t newval);
};

class StatCounterBank
{
public:
    friend bool operator==(const StatCounterBank &, const StatCounterBank &) = default;
    void clear() { count.clear(); rawTime=0; }
    int size() const { return count.size(); }
    double time() const { return rawTime * timeStep; }
    void setBits(int count_bits, int time_bits) { c_bits = count_bits; t_bits = time_bits; }
    void setTimeStep(double value) { timeStep = value; }
    quint64 operator[](int i) const { return count[i]; }
    template<typename T>
    void update(quint64 newRawTime, const QVector<T> &values) {
        if (timeWasReset(newRawTime)) {
            qDebug() << __PRETTY_FUNCTION__ << "reset detected";
            clear();
        }
        // rawTime = newRawTime;
        rawTime = StatCounter::update_with_rollover_fix(rawTime, newRawTime, t_bits);
        for (int i=0; i<values.size(); i++) {
            // count[i] = values[i];
            count[i] = StatCounter::update_with_rollover_fix(count[i], values[i], c_bits);
        }
    }
private:
    bool timeWasReset(quint64 newRawTime) const {
        return (newRawTime - rawTime)*timeStep > timeResetThreshold;
    }
    double timeResetThreshold = 60.0; // 1 minute
    int c_bits = 32;
    int t_bits = 32;
    QMap<int, quint64> count;
    quint64 rawTime = 0;
    double timeStep = 1;
};

#endif // STATCOUNTER_H
