#ifndef SDBTYPES_H
#define SDBTYPES_H

#include <QList>
#include <QMetaType>

#include "SdbData.h"

enum AFI_SDB_ID : uint32_t;

struct sdb_t
{
    sdb_t() = default;
    sdb_t(const sdb_t &r) = default;
    sdb_t &operator=(const sdb_t &r) = default;
    sdb_t(sdb_t &&r) = default;
    sdb_t &operator=(sdb_t &&r) = default;

    static sdb_t parse_rom_data(const struct sdb_rom_t &r);

    bool isValid() const;
    bool crcPresent() const;
    bool crcOk() const;

    SdbInterconnect ic() const;
    QVector<SdbDevice> devices() const;
    SdbSynthesis syn() const;

    QList<AFI_SDB_ID> getDeviceIds() const;

    bool hasSdbDevice(AFI_SDB_ID id) const;
    QPair<quint16, quint16> getSdbAddrRange(AFI_SDB_ID id, int seqNum=0) const;
    quint16 getSdbBaseAddr(AFI_SDB_ID id, int seqNum=0) const;
    QString getProductSignature() const;
    QList<SdbDevice> filterDevices(const QList<AFI_SDB_ID> &search) const;

    friend bool operator==(const sdb_t &, const sdb_t &) = default;

private:
    SdbData data;
    bool valid = false;
    bool crc_present = false;
    bool crc_valid = false;
};
Q_DECLARE_METATYPE(sdb_t)

#endif // SDBTYPES_H
