/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "BaseDeviceTableModel.h"

QMap<DeviceIndex, int> deviceIndexMap(const DeviceIndexEnabledVector &list)
{
   QMap<DeviceIndex, int> result;
   for (int i=0; i<list.size(); i++) {
       result.insert(list[i].index, i);
   }
   return result;
}

int BaseDeviceTableModel::rowCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    return devices.size();
}

int BaseDeviceTableModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    return COL_TOTAL;
}

DeviceIndex BaseDeviceTableModel::deviceIndexOf(int row) const
{
    auto it = devices.find(row);
    return it == devices.end() ? DeviceIndex() : it->index;
}

//DeviceIndexEnabled BaseDeviceTableModel::deviceIndexEnabled(const QModelIndex &index) const
//{
//    int row = index.row();
//    if (row < 0 || row >= devices.size())
//        return DeviceIndexEnabled();
//    return devices[row];
//}

int BaseDeviceTableModel::rowOf(const DeviceIndex &sourceIndex) const
{
    return devices.rowOf(sourceIndex);
}

//void BaseDeviceTableModel::setDevices(const DeviceTable &list)
//{
//    /*
//    // map deviceIndex to position in vector
//    const auto &old_map = deviceIndexMap(devices);
//    const auto &new_map = deviceIndexMap(list);
//    const auto &old_keys = old_map.keys();
//    const auto &new_keys = new_map.keys();
//    const auto &old_set = QSet<DeviceIndex>(old_keys.begin(), old_keys.end());
//    const auto &new_set = QSet<DeviceIndex>(new_keys.begin(), new_keys.end());
//    const auto &toAdd = new_set - old_set;
//    const auto &toRemove = old_set - new_set;
//    const auto &toChange = old_set & new_set;
//    // add new rows
//    if (! toAdd.isEmpty()) {
//        const auto &lastIndex = index(rowCount()-1, 1);
//        beginInsertRows(lastIndex, rowCount(), rowCount());
//        for (const auto &i : toAdd) {
//            devices.push_back(list[new_map.value(i)]);
//        }
//        endInsertRows();
//    }
//    // remove rows
//    if (! toRemove.isEmpty()) {
//        beginRemoveRows();
//        endRemoveRows();
//    }
//    // change rows
//    if (! toChange.isEmpty()) {
//        // check if modified
//        if (1) {
//            // ...
//            emit dataChanged(, , );
//        }
//    }
//    */
//    if (list == devices)
//        return;
//    beginResetModel();
//    devices = list; // FIXME: optimize it later
//    endResetModel();
//}

void BaseDeviceTableModel::addStaticDevices(const DeviceTable &list)
{
    for (const auto &i : list)
        addStaticDevice(i);
}

void BaseDeviceTableModel::setDynamicFlag(const DeviceIndex &index, DeviceTableEntry::DynamicFlag flag)
{
    for (auto &d : devices) {
        if (d.index == index) {
            d.dynamic = flag;
        }
    }
    invalidateRow(index);
}

void BaseDeviceTableModel::invalidateRow(const DeviceIndex &index)
{
    int row = devices.rowOf(index);
    if(row == -1)
        return;

    auto indexFrom = createIndex(row, 0);
    auto indexTo = createIndex(row, columnCount()-1);
    emit dataChanged(indexFrom, indexTo);
}

void BaseDeviceTableModel::addStaticDevice(const DeviceTableEntry &i)
{
    for (auto &d : devices) {
        if (d.index == i.index) {
            d.dynamic = DeviceTableEntry::Static;
            d.enabled = i.enabled;
            invalidateRow(d.index);
            return;
        }
    }
    beginInsertRows(QModelIndex(), rowCount(), rowCount());
    devices.insertStatic(i);
    endInsertRows();
}

//DeviceIndexEnabled BaseDeviceTableModel::removeDevice(const DeviceIndex &index)
//{
////    int row = deviceIndexRow(index);
////    if (row < 0)
////        return DeviceIndexEnabled(index);
//    DeviceIndex j;
//    DeviceIndexEnabled ret(j);
//    bool inResetMode = false;
//    for(int i=0; i<devices.size(); ++i) {
//        auto &d = devices[i];
//        if(d.index != index)
//            continue;
//        inResetMode = true;
//        beginResetModel();
//        devices.remove(i);
//        ret = d;
//        --i;
//    }
//    if(inResetMode)
//        endResetModel();
//    return ret;
//}

//DeviceIndexEnabled BaseDeviceTableModel::removeDevice(const QModelIndex &index)
//{
//    auto row = index.row();
//    if (row < 0 || row >= devices.size())
//        return DeviceIndexEnabled();
//    beginRemoveRows(QModelIndex(), row, row);
//    auto dev = devices.take(row);
//    endRemoveRows();
//    return dev;
//}

DeviceTable BaseDeviceTableModel::getDevices() const {
    return  devices;
}

DeviceIndexSet BaseDeviceTableModel::getDeviceIndices() const
{
    return devices.keys();
}

//DeviceIndexEnabled BaseDeviceTableModel::getDevice(int row) const
//{
//    return devices[row];
//}


//DeviceIndexEnabledMap BaseDeviceTableModel::getStaticDevices() const
//{
//    return devices.staticToMap();
//}

//DeviceIndexEnabledVector BaseDeviceTableModel::getEnabledDevices() const
//{
//    return toVector(filterEnabled(true));
//}

QMap<int, bool> BaseDeviceTableModel::defaultColumnState() const
{
    return {
        // {COL_ENABLE, false},
        // {COL_ONLINE, false},
        {COL_DYNAMIC, false},
        {COL_DEVICE_INDEX, false},
    };
}
