/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "IpPortPair.h"

#include <QHostAddress>

// parse "ip:port" string
IpPortPair IpPortPair::parse(const QString &s)
{
    IpPortPair p;
    const auto &list = s.split(":");
    if (list.size() >= 1) {
        bool Ok = false;
        auto value = list[0].toUInt(&Ok);
        if (Ok) {
            p.ip = value;
        } else {
            auto h = QHostAddress(list[0]);
            if(!h.isNull())
                p.ip = h.toIPv4Address();
        }

    }
    if (list.size() >= 2) {
        bool Ok = false;
        auto value = list[1].toUShort(&Ok);
        if (Ok)
            p.port = value;
    }
    return p;
}

QString IpPortPair::toString() const
{
    QString s = QHostAddress(ip).toString();
    if (port)
        s += ":" + QString::number(port);
    return s;
}
