/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef PROXYCHAIN_H
#define PROXYCHAIN_H

#include <QAbstractProxyModel>

#include "base/DeviceIndex.h"

class DeviceDynamicStateFilterProxyModel;
class DeviceIdFilter;
class DeviceIdentityProxyModel;
class DeviceIndexFilterProxyModel;
class Ipv4PrefixList;
class NetworkFilterProxyModel;
class QSortFilterProxyModel;

class ProxyChain : public QAbstractProxyModel
{
    Q_OBJECT

public:
    enum DeviceIdFilterMode {DisableDeviceIdFilter, EnableDeviceIdFilter};
    enum DeviceIndexFilterMode {NormalDeviceFilter, InverseDeviceFilter, DisableDeviceFilter};
    enum DeviceStateFilterMode {StaticDeviceStateFilter, DynamicDeviceStateFilter, DisableDeviceStateFilter};
    enum NetworkFilterMode {DisableNetworkFilter, EnableNetworkFilter};
    explicit ProxyChain(QObject *parent = nullptr);
    QAbstractProxyModel *createProxyChain(
        QAbstractItemModel *model, DeviceIdFilterMode deviceIdFilterMode,
        DeviceIndexFilterMode deviceIndexFilterMode,
        DeviceStateFilterMode deviceStateFilterMode,
        NetworkFilterMode networkFilterMode);
    static QModelIndex mapToDataSource(const QAbstractItemModel *model, const QModelIndex &index);
    NetworkFilterProxyModel* getNnetworkFilter() const { return networkFilter; }

    // reimplemented
    QModelIndex index(int row, int column, const QModelIndex& parent) const override;
    QModelIndex mapFromSource(const QModelIndex &sourceIndex) const override;
    QModelIndex mapToSource(const QModelIndex &proxyIndex) const override;
    QModelIndex parent(const QModelIndex& child) const override;
    int columnCount(const QModelIndex& parent) const override;
    int rowCount(const QModelIndex& parent) const override;

signals:
    void setDeviceIdFilter(const DeviceIdSet &r);
    void setDeviceIndexFilter(const DeviceIndexSet &r);
    void setNetworkFilter(const Ipv4PrefixList &filter);

private:
    DeviceIdFilter * deviceIdFilter = nullptr;
    DeviceIndexFilterProxyModel * deviceIndexFilter = nullptr;
    DeviceDynamicStateFilterProxyModel * deviceDynamicStateFilter = nullptr;
    NetworkFilterProxyModel * networkFilter = nullptr;
    QSortFilterProxyModel * sortProxy = nullptr;
    DeviceIdentityProxyModel * identityProxy = nullptr;
};

#endif // PROXYCHAIN_H
