/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "LldpDiscoveryDeviceListFilter.h"
#include "LldpDiscovery.h"

void LldpDiscoveryDeviceListFilter::lldpCacheUpdated(const LldpInfoCache &input)
{
    refreshCache(input);
}

void LldpDiscoveryDeviceListFilter::setDeviceList(const DeviceIndexEnabledMap &newFilter)
{
    filter = newFilter;
    refreshCache();
}

LldpInfoCache LldpDiscoveryDeviceListFilter::filterData(const LldpInfoCache &input) const
{
    LldpInfoCache result;
    for (auto it = input.cbegin(); it != input.cend(); ++it) {
        const auto &deviceIndex = it.key();
        auto found = filter.find(deviceIndex);
        if (found == filter.end())
            continue;
        if (! found.value().enabled)
            continue;
        result.insert(it.key(), it.value());
    }
    return result;
}

void LldpDiscoveryDeviceListFilter::refreshCache(const LldpInfoCache &input)
{
    dataCache.insert(input); // keep expired entries forever in dataCache
    auto result = filterData(dataCache);
    if (result == lastResult)
        return;
    lastResult = result;
    emit lldpCacheUpdatedFiltered(result);
}
