#include "AdcMpdTrigConfigConverter.h"

#include "AdcMpdTrigConfig.h"

namespace {
// AdcMpdTrigConfig
constexpr const char *k_groups = "groups";
constexpr const char *k_adcStaticCompensation = "adcStaticCompensation";

// AdcMpdTrigGroupConfig
constexpr const char *k_channels = "channels";
constexpr const char *k_trigThr = "trigThr";

// AdcMpdTrigChConfig
constexpr const char *k_en = "en";
constexpr const char *k_pol = "pol";
constexpr const char *k_adcCh = "adcCh";
}

// -------------- AdcMpdTrigChConfig
template<>
bool AbstractConfigConverter<AdcMpdTrigChConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<AdcMpdTrigChConfig>::toJsonObject(const AdcMpdTrigChConfig &in)
{
    QVariantMap m;
    m[k_en] = QVariant::fromValue(in.en);
    m[k_pol] = QVariant::fromValue(in.pol);
    m[k_adcCh] = QVariant::fromValue(in.adcCh);
    return fromVariantMap(m);
}

template<>
AdcMpdTrigChConfig AbstractConfigConverter<AdcMpdTrigChConfig>::fromJsonObject(const QJsonObject &in)
{
    AdcMpdTrigChConfig out;
    update_value(in, k_en, &out.en);
    update_value(in, k_pol, &out.pol);
    update_value(in, k_adcCh, &out.adcCh);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<AdcMpdTrigChConfig>::toJsonValue(const AdcMpdTrigChConfig &in)
{
    return toJsonObject(in);
}

template<>
AdcMpdTrigChConfig AbstractConfigConverter<AdcMpdTrigChConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<AdcMpdTrigChConfig>::registerSubTypes()
{
}

// -------------- AdcMpdTrigGroupConfig
template<>
bool AbstractConfigConverter<AdcMpdTrigGroupConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<AdcMpdTrigGroupConfig>::toJsonObject(const AdcMpdTrigGroupConfig &in)
{
    QVariantMap m;
    m[k_channels] = ConfigConverterUtil::toJsonValue(in.channels);
    m[k_trigThr] = QVariant::fromValue(in.trigThr);

    return fromVariantMap(m);
}

template<>
AdcMpdTrigGroupConfig AbstractConfigConverter<AdcMpdTrigGroupConfig>::fromJsonObject(const QJsonObject &in)
{
    AdcMpdTrigGroupConfig out;
    out.channels = ConfigConverterUtil::toVector<AdcMpdTrigChConfig>(in[k_channels]);
    update_value(in, k_trigThr, &out.trigThr);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<AdcMpdTrigGroupConfig>::toJsonValue(const AdcMpdTrigGroupConfig &in)
{
    return toJsonObject(in);
}

template<>
AdcMpdTrigGroupConfig AbstractConfigConverter<AdcMpdTrigGroupConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<AdcMpdTrigGroupConfig>::registerSubTypes()
{
    AbstractConfigConverter<AdcMpdTrigChConfig>::registerTypes();
}

// -------------- AdcMpdTrigConfig
template<>
bool AbstractConfigConverter<AdcMpdTrigConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<AdcMpdTrigConfig>::toJsonObject(const AdcMpdTrigConfig &in)
{
    QVariantMap m;
    m[k_groups] = ConfigConverterUtil::toJsonValue(in.groups);
    m[k_adcStaticCompensation] = ConfigConverterUtil::toJsonValue(in.adcStaticCompensation);
    return fromVariantMap(m);
}

template<>
AdcMpdTrigConfig AbstractConfigConverter<AdcMpdTrigConfig>::fromJsonObject(const QJsonObject &in)
{
    AdcMpdTrigConfig out;
    out.groups = ConfigConverterUtil::toVector<AdcMpdTrigGroupConfig>(in[k_groups]);
    out.adcStaticCompensation = ConfigConverterUtil::toVector<qint16>(in[k_adcStaticCompensation]);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<AdcMpdTrigConfig>::toJsonValue(const AdcMpdTrigConfig &in)
{
    return toJsonObject(in);
}

template<>
AdcMpdTrigConfig AbstractConfigConverter<AdcMpdTrigConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<AdcMpdTrigConfig>::registerSubTypes()
{
    AbstractConfigConverter<AdcMpdTrigGroupConfig>::registerTypes();
}
