#include "BmcModule.h"

#include "base/DeviceId.h"
#include "bmc/BmcStatus.h"
#include "mregdev/qtmregdevice.h"
#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

BmcModule::BmcModule(const MregBusInterface &bus)
    : BaseDeviceModule(bus)
{
    if (!isAvailable()) {
        if (getDeviceId() == DEVICE_ID_PHOS_FEC)
            setBaseAddr(0x6000, 0x00FF);
        else
            setBaseAddr(0x4000, 0x001F);
    }
}

QList<AFI_SDB_ID> BmcModule::getSupportedSdbIds()
{
    return {AFI_SDB_MCU};
}

void BmcModule::post_init()
{
    if (!isAvailable()) {
        setBaseAddr(0x4000, 0x1F);
    }
}

std::optional<BmcStatus> BmcModule::readStatus()
{
    if (!isAvailable())
        return {};
    QVector<quint16> result;
    auto regOp = regOpDumpAll(&result);
    if (!regOpExecRebased(regOp))
        return {};

    QMap<quint16, quint16> bmcRegisters;
    for (int i=0; i<result.size(); i++) {
        bmcRegisters[i] = result[i];
    }
    const auto pcb_ver = bus.dev->getDeviceHwVersion();
    return BmcStatus(getDeviceId(), pcb_ver, bmcRegisters);
}
