/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "FirFilterConfigConverter.h"
#include "FirFilter/FirFilterConfig.h"

constexpr const char *k_en = "en";
constexpr const char *k_roundoff = "roundoff";
constexpr const char *k_coef = "coef";

template<>
bool AbstractConfigConverter<FirFilterConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<FirFilterConfig>::toJsonObject(const FirFilterConfig &in)
{
    QVariantMap m;
    m[k_en] = QVariant::fromValue(in.en);
    m[k_roundoff] = QVariant::fromValue(in.roundoff);
    m[k_coef] = ConfigConverterUtil::toJsonValue(in.coef);
    return fromVariantMap(m);
}

template<>
FirFilterConfig AbstractConfigConverter<FirFilterConfig>::fromJsonObject(const QJsonObject &in)
{
    FirFilterConfig out;
    update_value(in, k_en, &out.en);
    update_value(in, k_roundoff, &out.roundoff);
    out.coef = ConfigConverterUtil::toVector<qint16>(in[k_coef]);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<FirFilterConfig>::toJsonValue(const FirFilterConfig &in)
{
    return toJsonObject(in);
}

template<>
FirFilterConfig AbstractConfigConverter<FirFilterConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<FirFilterConfig>::registerSubTypes()
{
}
