#include "MscHistModule.h"
#include "MscHistConfig.h"

#include "regio/RegOpStream.h"
#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

namespace {
enum {
    //Histograms CSR
    REG_CTRL = 0x0,
    REG_NUMBER_OF_RAM_BITS_HIST_TIME = 0x1,
    REG_NUMBER_OF_RAM_BITS_HIST_PLOT = 0x2,
    REG_BASE_HIST_PLOT_START_ADDR = 0x100,
    REG_HIST_PLOT = 0x10,
    REG_HIST_PLOT_CUR_RAM_ADDR = 0x2,               // 32-bit RO
};

enum {
    REG_CTRL_BIT_CNT_LOCK = 1 << 1,
    REG_CONTROL_LOCK_SOFTCLEAR = 1 << 0
};
}

static const int MAX_HIST_COUNT = 32;

QList<AFI_SDB_ID> MscHistModule::getSupportedSdbIds()
{
    return {AFI_SDB_MSC_HISTOGRAMS};
}

void MscHistModule::post_init()
{
    BaseDeviceModule::post_init();

    buildParams = readBuildParam();
}

mlink::RegOpVector MscHistModule::setConfigRegOp(const MscHistConfig &hist)
{
    RegOpVector r;
    for (int hId : hist.keys()) {
        r.RegWrite32(REG_BASE_HIST_PLOT_START_ADDR + REG_HIST_PLOT * hId,
                     hist[hId].plotSlice * 1e3);
    }
    return r;
}

bool MscHistModule::writeConfig(const MscHistConfig &hist)
{
    auto r = setConfigRegOp(hist);
    return regOpExecRebased(r);
}

int MscHistModule::getHistCount() const
{
    if (!histNum.has_value())
        return 0;
    int n = histNum.value();
    std::clamp(n, 0, MAX_HIST_COUNT);
    return n;
}

RegOpVector MscHistModule::getStatusRegOp(MscHistStatus *result) const
{
    RegOpVector r;
    r.RegWrite16(REG_CTRL, REG_CTRL_BIT_CNT_LOCK); // lock counters
    const auto histCount = getHistCount();
    result->plot_current_ram_address.resize(histCount);
    for (uint16_t i = 0; i < histCount; ++i) {
        r.RegRead16(REG_BASE_HIST_PLOT_START_ADDR + REG_HIST_PLOT * i + REG_HIST_PLOT_CUR_RAM_ADDR,
                       &result->plot_current_ram_address[i]);
    }

    r.RegWrite16(REG_CTRL, 0); // unlock counters
    return r;
}

std::optional<MscHistBuildParams> MscHistModule::readBuildParam()
{
    MscHistBuildParams bp;
    RegOpVector r;
    r.RegRead16(REG_NUMBER_OF_RAM_BITS_HIST_TIME, &bp.ramBitsHistTimeNum);
    r.RegRead16(REG_NUMBER_OF_RAM_BITS_HIST_PLOT, &bp.ramBitsHistPlotNum);

    if (!regOpExecRebased(r))
        return {};
    return bp;
}

bool MscHistModule::readStatus(MscHistStatus *result)
{
    auto r = getStatusRegOp(result);
    return regOpExecRebased(r);
}

std::optional<MscHistStatus> MscHistModule::readStatus()
{
    MscHistStatus result;
    if (readStatus(&result))
        return result;
    return {};
}
