/*
**    Copyright 2014-2018 Ivan Filippov
**    Copyright 2014-2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef RCTRIGCONFIG_H
#define RCTRIGCONFIG_H

#include <QMetaType>
#include <QtGlobal>

class QDebug;

class RcTrigConfig
{
public:
    enum TrigMask {
        TrigMaskTtl_1 = 0x0001,TrigMaskTtl_2 = 0x0002,
        TrigMaskTtl_3 = 0x0004,TrigMaskTtl_4 = 0x0008,
        TrigMaskTimer = 0x8000, TrigMaskRandom = 0x4000, TrigMaskLvds = 0x2000};
    bool operator==(const RcTrigConfig &ref) const;
    static double getClockFreqHz() { return 62500000; }
    static double getTrigTimerMinFreqHz() { return getClockFreqHz() / (quint32)(-1); }
    static double getTrigTimerMaxFreqHz() { return getClockFreqHz() / 2; }

    quint16 xOffMask = 0;
    bool trigOpen = true;
    quint16 trigSrc = TrigMaskTtl_1;
    double trigTimerFreq = 1000.0;  // Hz
    double trigRandomFreq = 1000.0; // Hz
    double trigRandomDeadTimeUs = 10.0; // us
};
Q_DECLARE_METATYPE(RcTrigConfig)

QDebug operator <<(QDebug stream, const RcTrigConfig &trig);
QDataStream &operator<<(QDataStream &out, const RcTrigConfig &r);
QDataStream &operator>>(QDataStream &in, RcTrigConfig &r);
#endif // RCTRIGCONFIG_H
