#ifndef RUNSTATISTIC_H
#define RUNSTATISTIC_H

#include <QtCore>

struct RunStatistic
{
    using Counters = std::array<quint64, 4>;
    using Timers = std::array<quint64, 7>;
    enum CounterIndex {
        Accepted = 0,
        Blocked = 1,
        Vetoed = 2,
        Discarded = 3
    };
    enum TimerIndex {
        TimerTotal = 0,
        TimerRun = 1,
        TimerBusyTotal = 2,
        TimerBusyAdc = 3,
        TimerBusyTdc = 4,
        TimerBusyReadout = 5,
        TimerExtVeto = 6
    };
    RunStatistic& operator-=(const RunStatistic& r) {
        for (size_t i = 0; i < counters.size(); i++)
            counters[i] -= r.counters[i];
        for (size_t i = 0; i < timers.size(); i++)
            timers[i] -= r.timers[i];
        return *this;
    }
    friend RunStatistic operator-(RunStatistic lhs, const RunStatistic& rhs) {
        lhs -= rhs;
        return lhs;
    }

    friend bool operator==(const RunStatistic &, const RunStatistic &) = default;
    void clear() { *this = {}; }

    enum ReportMode {Brief = false, Full = true};
    QString getText(ReportMode full) const;
    QString getTimerStr(TimerIndex index) const;
    double getTimer(TimerIndex index) const;
    quint64 getCounter(CounterIndex index) const;
    quint64 getRate(CounterIndex index) const;
    bool isValid() const;
    double clockFreqHz() const { return timerClock; }

    quint32 id = 0;
    Counters counters;
    Timers timers;
    quint32 timerClock = 125'000'000;
};

#endif // RUNSTATISTIC_H
