/*
**    Copyright 2014-2016 Ivan Filippov
**    Copyright 2014-2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef SPILLCONFIG_H
#define SPILLCONFIG_H

#include <QMetaType>
#include <QtGlobal>

class QDebug;

struct SpillConfig
{
    enum spill_sel_t {SPILL_OFF=0, SPILL_TIMER=1, SPILL_TTL=2, SPILL_WR=3, SPILL_INF=4, SPILL_AUTO=7, SPILL_GATE=8};

    friend bool operator==(const SpillConfig &, const SpillConfig &) = default;
    bool isSpillGate() const {
        return source == SpillConfig::SPILL_GATE;
    }

    quint32 durPhase1 = 0;    // acceleration, ms
    quint32 durPhase2 = 0;    // prep, ms
    quint32 durPhase3 = 1000; // readout, ms
    quint32 durPhase4 = 1;    // post-readout, ms
    quint32 durPhase5 = 1000; // flush, ms
    spill_sel_t source = SPILL_AUTO;
//    bool modeAuto;
    bool softSpill = false;
    quint32 timerPeriod = 7000; // in ms
};

Q_DECLARE_METATYPE(SpillConfig)

QDebug operator <<(QDebug stream, const SpillConfig &spill);
QDataStream &operator<<(QDataStream &out, const SpillConfig &r);
QDataStream &operator>>(QDataStream &in, SpillConfig &r);

#endif // SPILLCONFIG_H
