#include "TluTtlIoControlConfig.h"

#include <QMap>
#include <QList>

namespace {
static const QMap<TluTtlIoInput, QString> TluTtlIoInput_Names = {
    {TluTtlIoInput::Trigger, "Trigger"},
    {TluTtlIoInput::Veto, "Busy"},
    {TluTtlIoInput::LTR_0, "LTR[0]"},
    {TluTtlIoInput::LTR_1, "LTR[1]"},
    {TluTtlIoInput::LTR_2, "LTR[2]"},
    {TluTtlIoInput::LTR_3, "LTR[3]"},
    };

static const QMap<TluTtlIoInput, QString> TluTtlIoInput_PrettyNames = {
    {TluTtlIoInput::Trigger, "Trigger"},
    {TluTtlIoInput::Veto, "Busy"},
    {TluTtlIoInput::LTR_0, "Luminosity trigger #0"},
    {TluTtlIoInput::LTR_1, "Luminosity trigger #1"},
    {TluTtlIoInput::LTR_2, "Luminosity trigger #2"},
    {TluTtlIoInput::LTR_3, "Luminosity trigger #3"},
    };

static const QMap<TluTtlIoOutput, QString> TluTtlIoOutput_Names = {
    {TluTtlIoOutput::AcceptedTrigger, "AcceptedTrig"},
    {TluTtlIoOutput::RejectedTrigger, "RejectedTrig"},
    {TluTtlIoOutput::Busy, "Busy"},
    {TluTtlIoOutput::TOF_detector_trig, "TOF_trig"},
    {TluTtlIoOutput::Luminosity_detector_trig, "Lum_trig"},
    {TluTtlIoOutput::Pps, "Pps"},
    {TluTtlIoOutput::Logic0, "Logic0"},
    {TluTtlIoOutput::Logic1, "Logic1"}
};

static const QMap<TluTtlIoOutput, QString> TluTtlIoOutput_PrettyNames = {
    {TluTtlIoOutput::AcceptedTrigger, "Accepted trigger"},
    {TluTtlIoOutput::RejectedTrigger, "Rejected trigger"},
    {TluTtlIoOutput::Busy, "Busy"},
    {TluTtlIoOutput::TOF_detector_trig, "TOF detector trigger"},
    {TluTtlIoOutput::Luminosity_detector_trig, "Luminosity detector trigger"},
    {TluTtlIoOutput::Pps, "Pulse per second (WR or Fe-Link)"},
    {TluTtlIoOutput::Logic0, "Logic 0"},
    {TluTtlIoOutput::Logic1, "Logic 1"}
};

static const QList<TluTtlIoInput> TluTtlIoInput_List = TluTtlIoInput_Names.keys();
static const QList<TluTtlIoOutput> TluTtlIoOutput_List = TluTtlIoOutput_Names.keys();
}

void reg_tlu_ttl_io_t::setInput(TluTtlIoInput i)
{
    bit.isOutput = 0;
    bit.cfg = static_cast<int>(i);
}

void reg_tlu_ttl_io_t::setOutput(TluTtlIoOutput o)
{
    bit.isOutput = 1;
    bit.cfg = static_cast<int>(o);
}

TluTtlIoInput reg_tlu_ttl_io_t::getInput() const
{
    Q_ASSERT(bit.isOutput == 0);
    return static_cast<TluTtlIoInput>(bit.cfg);
}

TluTtlIoOutput reg_tlu_ttl_io_t::getOutput() const
{
    Q_ASSERT(bit.isOutput == 1);
    return static_cast<TluTtlIoOutput>(bit.cfg);
}

QString reg_tlu_ttl_io_t::toString() const
{
    return bit.isOutput ? TluTtlIoControlConfig::getName(getOutput()) : TluTtlIoControlConfig::getName(getInput());
}

QList<TluTtlIoInput> TluTtlIoControlConfig::getInputs()
{
    return TluTtlIoInput_List;
}

QList<TluTtlIoOutput> TluTtlIoControlConfig::getOutputs()
{
    return TluTtlIoOutput_List;
}

QString TluTtlIoControlConfig::getName(const TluTtlIoInput &type)
{
    return TluTtlIoInput_Names.value(type);
}

QString TluTtlIoControlConfig::getName(const TluTtlIoOutput &type)
{
    return TluTtlIoOutput_Names.value(type);
}

QString TluTtlIoControlConfig::getPrettyName(const TluTtlIoInput &type)
{
    auto it = TluTtlIoInput_PrettyNames.constFind(type);
    if(it != TluTtlIoInput_PrettyNames.cend())
        return *it;
    return TluTtlIoInput_Names.value(type);
}

QString TluTtlIoControlConfig::getPrettyName(const TluTtlIoOutput &type)
{
    auto it = TluTtlIoOutput_PrettyNames.constFind(type);
    if(it != TluTtlIoOutput_PrettyNames.cend())
        return *it;
    return TluTtlIoOutput_PrettyNames.value(type);
}
