/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "TtvxsCoreConfigConverter.h"
#include "base/AbstractConfigConverter.h"
#include "device-modules/Ttvxs/TtvxsCoreConfig.h"

namespace {
// reg_trig_src
constexpr const char *k_ttl = "ttl";
constexpr const char *k_sfp = "sfp";
constexpr const char *k_random_pulser = "random_pulser";
constexpr const char *k_fixed_pulser = "fixed_pulser";
// TtvxsBusySourceConfig
constexpr const char *k_vxsPulse = "vxsPulse";
constexpr const char *k_vxsFeLink = "vxsFeLink";
// TtvxsConfig
constexpr const char *k_trig_open = "trig_open";
constexpr const char *k_trigSrc = "trigSrc";
constexpr const char *k_vxsChMask = "vxsChMask";
constexpr const char *k_aux_cnt_for_trig = "aux_cnt_for_trig";
constexpr const char *k_busySource = "busySource";
} // namespace

// ============ reg_trig_src
template<>
bool AbstractConfigConverter<reg_trig_src>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<reg_trig_src>::toJsonObject(const reg_trig_src &in)
{
    QVariantMap m;
    m[k_ttl] = QVariant::fromValue(in.bit.ttl);
    m[k_sfp] = QVariant::fromValue(in.bit.sfp);
    m[k_random_pulser] = QVariant::fromValue(in.bit.random_pulser);
    m[k_fixed_pulser] = QVariant::fromValue(in.bit.fixed_pulser);
    return fromVariantMap(m);
}

template<>
reg_trig_src AbstractConfigConverter<reg_trig_src>::fromJsonObject(const QJsonObject &in)
{
    reg_trig_src out;
    out.bit.ttl = in[k_ttl].toInt();
    out.bit.sfp = in[k_sfp].toInt();
    out.bit.random_pulser = in[k_random_pulser].toInt();
    out.bit.fixed_pulser = in[k_fixed_pulser].toInt();
    return out;
}

template<>
QJsonValue AbstractConfigConverter<reg_trig_src>::toJsonValue(const reg_trig_src &in)
{
    return toJsonObject(in);
}

template<>
reg_trig_src AbstractConfigConverter<reg_trig_src>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<reg_trig_src>::registerSubTypes()
{
}

// ============ TtvxsBusySourceConfig

template<>
bool AbstractConfigConverter<TtvxsBusySourceConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<TtvxsBusySourceConfig>::toJsonObject(const TtvxsBusySourceConfig &in)
{
    QVariantMap m;
    m[k_vxsPulse] = QVariant::fromValue(in.vxsPulse);
    m[k_vxsFeLink] = QVariant::fromValue(in.vxsFeLink);
    return fromVariantMap(m);
}

template<>
TtvxsBusySourceConfig AbstractConfigConverter<TtvxsBusySourceConfig>::fromJsonObject(const QJsonObject &in)
{
    TtvxsBusySourceConfig out;
    update_value(in, k_vxsPulse, &out.vxsPulse);
    update_value(in, k_vxsFeLink, &out.vxsFeLink);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<TtvxsBusySourceConfig>::toJsonValue(const TtvxsBusySourceConfig &in)
{
    return toJsonObject(in);
}

template<>
TtvxsBusySourceConfig AbstractConfigConverter<TtvxsBusySourceConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<TtvxsBusySourceConfig>::registerSubTypes()
{
    AbstractConfigConverter<reg_trig_src>::registerTypes();
}
// ============ TtvxsCoreConfig

template<>
bool AbstractConfigConverter<TtvxsCoreConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<TtvxsCoreConfig>::toJsonObject(const TtvxsCoreConfig &in)
{
    QVariantMap m;
    m[k_trig_open] = QVariant::fromValue(in.trig_open);
    m[k_trigSrc] = QVariant::fromValue(in.trigSrc);
    m[k_vxsChMask] = QVariant::fromValue(in.vxsChMask);
    m[k_aux_cnt_for_trig] = QVariant::fromValue(in.aux_cnt_for_trig);
    m[k_busySource] = QVariant::fromValue(in.busySource);
    return fromVariantMap(m);
}

template<>
TtvxsCoreConfig AbstractConfigConverter<TtvxsCoreConfig>::fromJsonObject(const QJsonObject &in)
{
    TtvxsCoreConfig out;
    update_value(in, k_trig_open, &out.trig_open);
    update_value(in, k_trigSrc, &out.trigSrc);
    update_value(in, k_vxsChMask, &out.vxsChMask);
    update_value(in, k_aux_cnt_for_trig, &out.aux_cnt_for_trig);
    update_value(in, k_busySource, &out.busySource);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<TtvxsCoreConfig>::toJsonValue(const TtvxsCoreConfig &in)
{
    return toJsonObject(in);
}

template<>
TtvxsCoreConfig AbstractConfigConverter<TtvxsCoreConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<TtvxsCoreConfig>::registerSubTypes()
{
    AbstractConfigConverter<reg_trig_src>::registerTypes();
    AbstractConfigConverter<TtvxsBusySourceConfig>::registerTypes();
}
