/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef WAVEBLCMODULE_H
#define WAVEBLCMODULE_H

#include "WaveBlc/WaveBlcStatus.h"
#include "device-modules/BaseDeviceModule.h"
#include <optional>

class MultichannelBaselineStats;
class WaveBlcConfig;

class WaveBlcModule : public BaseDeviceModule<WaveBlcModule>
{
public:
    explicit WaveBlcModule(const MregBusInterface &bus);

    static QList<AFI_SDB_ID> getSupportedSdbIds();

    bool writeConfig(const WaveBlcConfig &config);

    bool readStatus(WaveBlcStatus *result);
    std::optional<WaveBlcStatus> readStatus();

    QMap<int, QVector<int>> getAllChannelData(int repeat = 1);
    bool readBaseline(MultichannelBaselineStats *result, int repeat = 1);

private:
    bool isV1_1() const;

    WaveBlcStatus status;
};

#endif // WAVEBLCMODULE_H
