/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ABSTRACTTHRESHOLDCALIBRATOR_H
#define ABSTRACTTHRESHOLDCALIBRATOR_H

#include "cal/CalResult.h"

#include <QMap>
#include <QVector>

class AbstractThresholdCalibrator
{
public:
    virtual ~AbstractThresholdCalibrator() = default;
    DacCalResult calibrateDac(int minRawValue, int maxRawValue, int step = 1);

protected:
    using AllChannelState = QVector<double>;

    virtual bool writeLevels(const QMap<int, int> &values) = 0;
    virtual bool readStates(AllChannelState *states) = 0;
    virtual double convert_raw_to_mV(double raw) const = 0;
    virtual int getNumberOfChannels() const = 0;

    static AllChannelState calcProbabilities(const QVector<quint64> &chStates, int nch);

private:
    using RawResult = QMap<int, int>;
    struct Workspace
    {
        Workspace(int nch) : nch(nch) {}
        RawResult makeResult();
        const int nch;
        QMap<quint16, AllChannelState> dacToChState;
    };
    AllChannelState searchStep(Workspace *workspace, int dacVal);
    RawResult runSearch(Workspace *workspace, int minRawValue, int maxRawValue, int step);
    int findNumberOfChannels();
};

#endif // ABSTRACTTHRESHOLDCALIBRATOR_H
