#ifndef ADCSERDESCALDIALOG_H
#define ADCSERDESCALDIALOG_H

#include <QDialog>

#include "device-modules/DeserializersControl/AdcSerDesDelay.h"
#include "fsm/FsmState.h"

class DeviceIndex;
class SingleDelayWidget;
class DeviceIndexEnabledMap;
struct DeserializersControlBuildParams;

namespace Ui {
class AdcSerDesCalDialog;
}

class AdcSerDesCalDialog : public QDialog
{
    Q_OBJECT
public:
    AdcSerDesCalDialog(QWidget *parent = nullptr);
    ~AdcSerDesCalDialog() override;

    // AdcSerDesDelayMap subscribe
signals: void devSerDesDelayChanged(const DeviceIndex &index, const AdcSerDesDelayMap &delay) const;
public slots: void setDevSerDesDelay(const DeviceIndex &index, const AdcSerDesDelayMap &delay);

    // DeviceIndexEnabledMap subscribe
signals: void deviceListChanged(const DeviceIndexEnabledMap &);
public slots: void setDeviceList(const DeviceIndexEnabledMap &new_devices);

signals:
    void requestCalibration(bool hardMode, QString = "");

public slots:
    void onFsmStateChange(const FsmState &state);
    void setDeserializersControlBp(const DeviceIndex &index, const DeserializersControlBuildParams &bp);
    void progressUpdated(const DeviceIndex &index, int progress);
    void onDeviceChange(const DeviceIndex &di);

private slots:
    // from Actions
    void on_comboBoxCurDev_currentIndexChanged(int);
    void on_groupBoxSaveCalRes_toggled(bool en);
    void on_lineEditPath_textChanged(const QString &);
    void on_pushButtonBrowse_clicked();
    void on_checkBoxHardMode_toggled(bool);
    void on_pushButtonCalibrate_clicked();

    //from SingleSerDesDelayWidget
    void onClockDelayChanged(int id, int val);
    void onDataDelayChanged(int id, int val);

private:
    void restoreGui();
    void resoreLocalSettings();
    void saveLocalSettings();
    void validateSaveFolder();

    Ui::AdcSerDesCalDialog *ui;

    DeviceIndexSet devSet;
    DevSerDesDelayMap allDev;
    DeviceIndex curDev;
    QMap<DeviceIndex, int> devProgress;
    QMap<DeviceIndex, int> devDesCnt;
    bool savePathVaild = true;
};

#endif // ADCSERDESCALDIALOG_H
