#ifndef DEVICECHMODEL_H
#define DEVICECHMODEL_H

#include <QAbstractItemModel>
#include <QModelIndex>
#include <QVariant>

#include "DeviceChModelItem.h"
#include "ModularDeviceStatus.h"
#include "TreeColumns.h"
#include "base/DeviceIndex.h"
#include "lib-common/BaseDeviceAppConfig.h"

class QTimer;
class ConfigKey;

class DeviceChModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    DeviceChModel(QObject *parent = nullptr);
    QVariant data(const QModelIndex &index, int role) const override;
    virtual bool setData(const QModelIndex &index,
                         const QVariant &value,
                         int role = Qt::EditRole) override;
    Qt::ItemFlags flags(const QModelIndex &index) const override;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role) const override;
    QModelIndex index(int row, int column,
                      const QModelIndex &parent = QModelIndex()) const override;
    QModelIndex parent(const QModelIndex &index) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override
    {
        Q_UNUSED(parent)
        return static_cast<int>(TreeColumns::COL_TOTAL);
    }

    void setEnabled(bool en);
    int getDevRow(const DeviceIndex &di) const;

signals:
    void configChanged(const BaseDeviceAppConfig &config) const;

public slots:
    void setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig);
    void deviceStatusUpdated(const ModularDeviceStatus &status);

private slots:
    void postCheckStateChange();

private:
    void enqueueConfigSend();
    void sendDataChanged(QVector<int> roles = QVector<int>());

    QVariant displayData(const QModelIndex &index) const;
    QVariant editData(const QModelIndex &index) const;
    QVariant checkStateData(const QModelIndex &index) const;
    QVariant fontData(const QModelIndex &index) const;

    inline TreeColumns getColumnByIndex(const QModelIndex &index) const;
    inline DeviceChModelItem *getItemByIndex(const QModelIndex &index) const;
    QVariant getRecursiveState(const QModelIndex &index, QMap<int, bool> TqdcCoreConfig::*ptr, bool defSt) const;
    bool enableColumn(const DeviceIndex &deviceIndex, TreeColumns col) const;
    bool hasDigTrig(const DeviceIndex i) const;
    bool hasZsDig(const DeviceIndex i) const;
    void recursiveDataChanged(const QModelIndex &index, const QVector<int> &roles);

    BaseDeviceAppConfig config;
    QMap<DeviceIndex, ModularDeviceStatus> dev_status;
    DeviceChModelItem rootItem;
    QTimer *const delayedSendConfigTimer;
    bool enable = true;
};

#endif // DEVICECHMODEL_H
