#include "DlnpCalibrationTriggerWidget.h"
#include "ui_DlnpCalibrationTriggerWidget.h"

#include "base/gui_utils.h"
#include "DlnpCalibrationTrigger/DlnpCalibrationTriggerBuildParams.h"
#include "DlnpCalibrationTrigger/DlnpCalibrationTriggerConfig.h"

class DlnpCalibrationTriggerWidgetPrivate
{
public:
    DlnpCalibrationTriggerConfig cfg;
    std::optional<DlnpCalibrationTriggerBuildParams> buildParams;
};

DlnpCalibrationTriggerWidget::DlnpCalibrationTriggerWidget(QWidget *parent)
    : QGroupBox(parent)
    , ui(new Ui::DlnpCalibrationTriggerWidget)
    , d_ptr(new DlnpCalibrationTriggerWidgetPrivate)
{
    ui->setupUi(this);
    restoreGui();
}

DlnpCalibrationTriggerWidget::~DlnpCalibrationTriggerWidget()
{
    delete ui;
}

void DlnpCalibrationTriggerWidget::setDlnpCalibrationTriggerConfig(const DlnpCalibrationTriggerConfig &c)
{
    Q_D(DlnpCalibrationTriggerWidget);
    if(d->cfg == c)
        return;
    d->cfg = c;
    restoreGui();
}

void DlnpCalibrationTriggerWidget::dlnpCalibrationTriggerBpUpdated(const DeviceIndex &index, const DlnpCalibrationTriggerBuildParams &bp)
{
    Q_UNUSED(index);
    Q_D(DlnpCalibrationTriggerWidget);
    if(d->buildParams && *d->buildParams == bp)
        return;
    d->buildParams = bp;
    restoreGui();
}

void DlnpCalibrationTriggerWidget::on_checkBoxEn_toggled(bool en)
{
    Q_D(DlnpCalibrationTriggerWidget);
    d->cfg.en = en;
    emit dlnpCalibrationTriggerConfigChanged(d->cfg);
}

void DlnpCalibrationTriggerWidget::on_spinBoxPeriod_valueChanged(int periodUs)
{
    Q_D(DlnpCalibrationTriggerWidget);
    if(!d->buildParams)
        return;
    d->cfg.period = static_cast<quint32>(1e-3 * periodUs * d->buildParams->clockKHz);
    emit dlnpCalibrationTriggerConfigChanged(d->cfg);
}

void DlnpCalibrationTriggerWidget::on_spinBoxCount_valueChanged(int count)
{
    Q_D(DlnpCalibrationTriggerWidget);
    d->cfg.count = static_cast<quint16>(count);
    emit dlnpCalibrationTriggerConfigChanged(d->cfg);
}

void DlnpCalibrationTriggerWidget::on_spinBoxOffset_valueChanged(int offsetUs)
{
    Q_D(DlnpCalibrationTriggerWidget);
    if(!d->buildParams)
        return;
    d->cfg.offset = static_cast<quint32>(1e-3 * offsetUs * d->buildParams->clockKHz);
    emit dlnpCalibrationTriggerConfigChanged(d->cfg);
}

void DlnpCalibrationTriggerWidget::restoreGui()
{
    Q_D(const DlnpCalibrationTriggerWidget);
    setEnabled(d->buildParams && d->buildParams->isValid());
    if(!d->buildParams)
        return;
    RecursiveWidgetSignalBlocker blocker(this);
    ui->checkBoxEn->setChecked(d->cfg.en);
    ui->spinBoxCount->setValue(d->cfg.count);
    int periodUs = 1e3 * d->cfg.period / d->buildParams->clockKHz;
    int offsetUs = 1e3 * d->cfg.offset / d->buildParams->clockKHz;
    ui->spinBoxPeriod->setValue(periodUs);
    ui->spinBoxOffset->setValue(offsetUs);
}
