/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "NetworkPortConfigDialog.h"
#include "NetworkPortWidget.h"

#include <QGridLayout>
#include <QAbstractButton>
#include <QDialogButtonBox>

NetworkPortConfigDialog::NetworkPortConfigDialog(QWidget *parent)
    : QDialog(parent)
    , widget(new NetworkPortWidget(this))
{
    setModal(false);
    resize(800, 500);
    auto layout = new QGridLayout(this);
    layout->addWidget(widget);
    connect(widget, &NetworkPortWidget::mstreamReset,
            this, &NetworkPortConfigDialog::mstreamReset);
    //    connect(widget, &NetworkPortWidget::configChanged,
    //            this, &NetworkPortConfigDialog::networkPortConfigChanged);

    auto buttonBox = new QDialogButtonBox(this);
    buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
    buttonBox->setOrientation(Qt::Horizontal);
    buttonBox->setStandardButtons(QDialogButtonBox::Apply |
                                  QDialogButtonBox::RestoreDefaults |
                                  QDialogButtonBox::Close);
    layout->addWidget(buttonBox);
    connect(buttonBox, &QDialogButtonBox::accepted,
            this, &QDialog::accept);
    connect(buttonBox, &QDialogButtonBox::rejected,
            this, &QDialog::reject);
    for (auto *button : buttonBox->buttons()) {
        switch (buttonBox->standardButton(button)) {
        case QDialogButtonBox::Apply:
            connect(button, &QAbstractButton::clicked,
                    this, &NetworkPortConfigDialog::apply);
            break;
        case QDialogButtonBox::RestoreDefaults:
            connect(button, &QAbstractButton::clicked,
                    this, &NetworkPortConfigDialog::restoreDefaults);
            break;
        case QDialogButtonBox::Close:
            connect(button, &QAbstractButton::clicked,
                    this, &NetworkPortConfigDialog::discard);
            break;
        default:;
        }
    }
}

void NetworkPortConfigDialog::setNetworkPortConfig(const NetworkPortConfig &config)
{
    if (config == cache)
        return;
    cache = config;
    widget->setConfig(config);
}

void NetworkPortConfigDialog::apply()
{
    cache = widget->getConfig();
    emit networkPortConfigChanged(cache);
}

void NetworkPortConfigDialog::restoreDefaults()
{
    widget->setConfig({});
}

void NetworkPortConfigDialog::discard()
{
    widget->setConfig(cache);
}

void NetworkPortConfigDialog::mstreamReset()
{
    cache.mstreamReset = true;
    emit networkPortConfigChanged(cache);
    cache.mstreamReset = false;
    emit networkPortConfigChanged(cache);
}
