﻿/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "NetworkPortWidget.h"
#include "ui_NetworkPortWidget.h"

#include <QComboBox>
#include <QDebug>
#include <QLabel>
#include <QSizePolicy>

#include "base/Globals.h"
#include "base/gui_utils.h"
#include "device-modules/NetworkPort/NetworkPortConfig.h"
#include "device-modules/NetworkPort/NetworkPortStatus.h"

NetworkPortWidget::NetworkPortWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::NetworkPortWidget)
{
    ui->setupUi(this);

    RecursiveWidgetSignalBlocker blocker(this);
    ui->spinBoxMtu->setMinimum(NetworkPortConfig::getMtuMin());
    ui->spinBoxMtu->setMaximum(NetworkPortConfig::getMtuMax());
    ui->spinBoxDefragTimeout->setMinimum(NetworkPortConfig::getDefragTimeoutMin());
    ui->spinBoxDefragTimeout->setMaximum(NetworkPortConfig::getDefragTimeoutMax());
    ui->spinBoxRetxTimeout->setMinimum(NetworkPortConfig::getRetxTimeoutMin());
    ui->spinBoxRetxTimeout->setMaximum(NetworkPortConfig::getRetxTimeoutMax());
    updateGui();
}

NetworkPortWidget::~NetworkPortWidget()
{
    delete ui;
}

void NetworkPortWidget::setConfig(const NetworkPortConfig &newConfig)
{
    if (newConfig == cfg)
        return;
    cfg = newConfig;

    updateGui();
}

void NetworkPortWidget::updateGui()
{
    RecursiveWidgetSignalBlocker blocker(this);
    ui->spinBoxMtu->setValue(cfg.mtu);
    ui->spinBoxDefragTimeout->setValue(cfg.defragTimeoutUs);
    ui->spinBoxRetxTimeout->setValue(cfg.retxTimeoutUs);
}

void NetworkPortWidget::on_spinBoxMtu_valueChanged(int value)
{
    cfg.mtu = value;
    emit configChanged(cfg);
}

 void NetworkPortWidget::on_spinBoxDefragTimeout_valueChanged(int value)
{
     cfg.defragTimeoutUs = value;
     emit configChanged(cfg);
}


void NetworkPortWidget::on_spinBoxRetxTimeout_valueChanged(int value)
{
    cfg.retxTimeoutUs = value;
    emit configChanged(cfg);
}

void NetworkPortWidget::on_pushButtonResetMstream_clicked()
{
    emit mstreamReset();
}
