#ifndef SPARSEDIALOG_H
#define SPARSEDIALOG_H

#include <QDialog>
#include <QMap>

#include "base/DeviceIndex.h"
#include "AdcErc/AdcErcConfig.h"

struct AdcBuildParams;
struct DeviceIndexEnabledMap;

namespace Ui {
class SparseDialog;
}

class SparseDialog : public QDialog
{
    Q_OBJECT

public:
    explicit SparseDialog(QWidget *parent = nullptr);
    ~SparseDialog() override;

public slots:
    void setDeviceList(const DeviceIndexEnabledMap &new_devices);
    void deviceInfoUpdated(DeviceIndex index, const AdcBuildParams &par);
    void setSparseParams(const SparseParams &sparseParams);

signals:
    void deviceListChanged(const DeviceIndexEnabledMap &);
    void sparseParamsChanged(const SparseParams &config);
    void showSparseMarkersUpdated(bool val) const;

private Q_SLOTS:
    void on_checkBoxEnable_toggled(bool checked);
    void on_spinBoxReadCellNum_valueChanged(int arg1);
    void on_spinBoxPeriod_valueChanged(int arg1);
    void on_comboBoxPointsSel_currentIndexChanged(int index);

private:
    void restoreFormSettings();
    void setOffsetMaximum(const DeviceIndex &index, quint16 max);
    void blockOffset(const DeviceIndex &index, bool blockOffset);
    void upMaxSampleCnt() const;

    Ui::SparseDialog *const ui;
    SparseParams cfg;

    QMap<DeviceIndex, quint16> devMaxSampleCnt;
    DeviceIndexSet devSet;
    QSet<DeviceIndex> devBlockOffset;
};

#endif // SPARSEDIALOG_H
