#ifndef TELNETREMOTECONTROL_H
#define TELNETREMOTECONTROL_H

#include <QTcpServer>
#include <QTcpSocket>
#include <QMap>

#include "fsm/FsmState.h"
struct TelnetBlockedEvents;

class TelnetRemoteControl : public QObject
{
    Q_OBJECT
public:
    TelnetRemoteControl(quint16 port, QObject *parent = nullptr);
    virtual ~TelnetRemoteControl() override;

signals:
    void startRun() const;
    void stopRun() const;

public slots:
    void timedRunFinished(const TelnetBlockedEvents &evBloked);
    void onFsmStateChange(const FsmState &state);

private slots:
    void updateConnections();
    void readCommandRequest();
    void onDisconnected();
    void init();

protected:
    virtual void setParam(const QString &key, const QVariant &val);
    void sendWarn(const QString &s) const;
    void sendString(const QString &mess) const;
    void printHelp() const;

private:
    void decode();
    bool isRunControlConnected() const;
    void sendState() const;

    const quint16 tcpPort;
    QTcpServer * const qTcpServer;
    QTcpSocket * tcpSocket = nullptr;
    bool portInUse = false;

    QByteArray cachedData;
    FsmState fsmSt;
};

#endif // TELNETREMOTECONTROL_H
