/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "BaseDeviceAppConfigConverter.h"

#include "BaseDeviceAppConfig.h"
#include "base/ConfigConverterUtil.h"

namespace {
constexpr const char *k_knownSetups = "knownSetups";
constexpr const char *k_defaultSetup = "defaultSetup";
constexpr const char *k_allowRemoteControl = "AllowRemoteControl";
constexpr const char *k_devStatusZmqPublisher = "DevStatusZmqPublisher";
constexpr const char *k_errOnTrigOnXOff = "errOnTrigOnXOff";
}

template<>
bool AbstractConfigConverter<BaseDeviceAppConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<BaseDeviceAppConfig>::toJsonObject(const BaseDeviceAppConfig &in)
{
    QVariantMap m;
    m[k_knownSetups] = ConfigConverterUtil::toJsonValue(in.knownConfigs);
    m[k_defaultSetup] = QVariant::fromValue(in.defaultSetup);
    m[k_allowRemoteControl] = QVariant::fromValue(in.allowRemoteControl);
    m[k_devStatusZmqPublisher] = QVariant::fromValue(in.devStatusZmqPublisher);
    m[k_errOnTrigOnXOff] = QVariant::fromValue(in.errOnTrigOnXOff);
    return fromVariantMap(m);
}

template<>
BaseDeviceAppConfig AbstractConfigConverter<BaseDeviceAppConfig>::fromJsonObject(const QJsonObject &in)
{
    BaseDeviceAppConfig out;
    out.knownConfigs = ConfigConverterUtil::toMap<DeviceIndex, ModularDeviceConfig>(
        in[k_knownSetups]);
    update_value(in, k_defaultSetup, &out.defaultSetup);
    update_value(in, k_allowRemoteControl, &out.allowRemoteControl);
    update_value(in, k_devStatusZmqPublisher, &out.devStatusZmqPublisher);
    update_value(in, k_errOnTrigOnXOff, &out.errOnTrigOnXOff);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<BaseDeviceAppConfig>::toJsonValue(const BaseDeviceAppConfig &in)
{
    return toJsonObject(in);
}

template<>
BaseDeviceAppConfig AbstractConfigConverter<BaseDeviceAppConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<BaseDeviceAppConfig>::registerSubTypes()
{
    AbstractConfigConverter<DeviceIndex>::registerTypes();
    AbstractConfigConverter<ModularDeviceConfig>::registerTypes();
    AbstractConfigConverter<ZmqConfig>::registerTypes();
}
