/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef MSTREAMSTATISTICTABLEMODEL_H
#define MSTREAMSTATISTICTABLEMODEL_H

#include "mvc/DeviceIndexTableModel.h"

#include "base/DeviceIndexEnabled.h"
#include "mstream-lib/MStreamStat.h"
#include "mstream-lib/ReceiverStat.h"

class MStreamStatisticTable;

class MStreamStatisticTableModel : public DeviceIndexTableModel
{
    Q_OBJECT
public:
    enum Columns {
        COL_TYPE,                  COL_SERIAL,
        COL_CONNECTION,
        COL_PKG_COUNT,             COL_PKG_RATE,          COL_RECV_DATA_SIZE,
        COL_RECV_DATA_RATE,        COL_RETRANS_FRAG_PARTS,
        COL_FRAG_TOTAL,            COL_FRAG_CLOSED,       COL_FRAG_DROPED,
        COL_FRAG_MISSED,           COL_FRAG_INCOMPLETED,  COL_GOOD_FORMAT_PKG,
        COL_BAD_FORMAT_PKG,        COL_MISSED_EVENTS,     COL_MISSED_EVENTS_RANGES,
        COL_INCOMPLETE_EVENTS,     COL_LAST_EVENT_NUM,    COL_EVENT_COUNT,
        COL_EVENT_RATE,            COL_SLOW_CONN_PERCENT,

        COL_NO_FREE,               COL_DIDNT_HELP,        COL_FREE,
        COL_CHECK_FAIL,            COL_NO_DATA,

        COL_TOTAL
    };
    explicit MStreamStatisticTableModel(QObject *parent = nullptr);
    ~MStreamStatisticTableModel() override;

    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const override;

    virtual QVariant data(const QModelIndex &index, int role) const override;
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const override;
    DeviceIndex deviceIndexOf(int row) const override; // row_to_index
    int rowOf(const DeviceIndex &sourceIndex) const override; // index_to_row

    QMap<DeviceIndex, MStreamStat> getStat() const; // { return prev_ms_stat; }
    QMap<int, bool> defaultColumnState() const override;

public slots:
    void setDeviceList(const DeviceIndexEnabledMap &new_devices);
    void stat_updated(const MStreamStat &ms_stat);
    void stat_from_receiver_updated(const ReceiverStat &stat);
    void mstreamConnectionStateChanged(const DeviceIndex &deviceIndex, bool connectionState);

private:
    QVariant data_DisplayRole(const DeviceIndex &index, int column) const;
    QVariant data_BackgroundRole(const DeviceIndex &index, int column) const;
    QVariant data_ToolTipRole(const DeviceIndex &index, int column) const;

    // DeviceIndexSet devices;
    // QMap<DeviceIndex, MStreamStat> prev_ms_stat;
    MStreamStatisticTable * const devices;
};

#endif // MSTREAMSTATISTICTABLEMODEL_H
