/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef RCFSM_H
#define RCFSM_H

#include "fsm/BaseFsm.h"

struct RcFsmInput : public BaseFsmInput
{
    //    static QString toString(const RcFsmInput &r);

    FsmState trigDeviceMgrState = FsmState::Unknown;
    FsmState clientManagerRunState = FsmState::Unknown;
    FsmState vmeTrigsRunState = FsmState::Unknown;

    bool spillStateOld = false;
    bool spillStateCur = false;
};

class RcFsm: public BaseFsm
{
    Q_OBJECT

public:
    explicit RcFsm(QObject *parent = nullptr);
    virtual bool isTransitionAllowed(const FsmState &toState) const;

public slots:
    void setTrigDeviceMgrState(const FsmState &state);
    void setClientManagerRunState(const FsmState &state);
    void setVmeTrigsRunState(const FsmState &state);
    void setSpillState(bool inSpillState);

private slots:
    virtual bool evaluate() override;

protected:
    virtual void printInputString(QDebug dbg) const override;
    virtual qint64 stateTimeoutMs(const FsmState &state) const override;
    inline RcFsmInput *getInput() {
        return static_cast<RcFsmInput*>(BaseFsm::getInput());
    }
    inline const RcFsmInput *getInput() const {
        return static_cast<const RcFsmInput*>(BaseFsm::getInput());
    }

private:
    void setInputState(FsmState RcFsmInput::*p, const char *fieldName, const FsmState &state);
    bool allInState(FsmState state) const;
    bool anyError() const;
};

QDebug operator<<(QDebug dbg, const RcFsmInput &state);

#endif // RCFSM_H
