//
//    Copyright 2016 Ivan Filippov
//    Copyright 2016 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef TCP_METRIC_SENDER_H
#define TCP_METRIC_SENDER_H

#include <QObject>
#include <QHostAddress>
#include <QMap>
#include <QSemaphore>

#include "MetricTypes.h"

class MetricSenderHelper;
class QTcpSocket;
class QTimer;

class TcpMetricSender : public QObject
{
    Q_OBJECT
public:
    explicit TcpMetricSender(QObject *parent = nullptr);
    virtual ~TcpMetricSender();

    QSemaphore sem;

public slots:
    void setBaseTags(const MetricTags &s);
    void sendMetric(const MetricEntity &ent = MetricEntity());

private slots:
    void deleteClient();
    void reconnectToServer();
    void displayError(QAbstractSocket::SocketError socketError);
    void connected();
    void disconnected();
    void runQueue();

private:
    bool sendQueue();
    bool isConnectionAlive() const;

    MetricSenderHelper * const helper;
    QTcpSocket * const tcpSocket;
    QTimer * const reconnectTimer;
    QTimer * const runQueueTimer;
    int connectionAttempts;
    QStringList queue;
};

#endif // TCP_METRIC_SENDER_H
