#include "SdbReader.h"

#include <QDebug>
#include <vector>

#include "mregdev/MregDevice.h"
#include "regio/regop.h"
#include "sdb/SdbTypes.h"
#include "sdb/bswap.h"
#include "sdb/sdb_rom.h"

static const std::initializer_list<quint32>
    SDB_LOCATIONS {0x7C00, 0, 0x7800};

/// find SDB at fixed addresses
std::optional<uint32_t> SdbReader::find(mlink::MregDevice *dev)
{
    for (auto i : SDB_LOCATIONS) {
        mlink::RegOpVector r;
        quint32 id;
        r.RegRead32(0, &id);
        r.rebase(i);
        if (!dev->regOpExec(r))
            return {};
        if (id == ntohl(SDB_MAGIC)) {
            return i;
        }
    }
    return {};
}

std::optional<sdb_t> SdbReader::read(mlink::MregDevice *dev, uint32_t address)
{
    sdb_rom_t sdb_rom;
    const auto nreg = sizeof(sdb_rom_t) / 2;
    auto rawdata = reinterpret_cast<quint16 *>(&sdb_rom);
    mlink::RegOpVector r;
    for (size_t i=0; i<nreg; i++)
        r.RegRead16(i, &rawdata[i]);
    r.rebase(address);
    if (!dev->regOpExec(r))
        return  {};
    return sdb_t::parse_rom_data(sdb_rom);
}
