/*
**    Copyright 2011-2022 Ivan Filippov
**    Copyright 2011-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "BaseFragment.h"

#include <cstring> // memcpy

#include "types.h"

static const bool bigFragId = true;

void BaseFragment::init(const MStreamFragmentHeader *f)
{
    free = false;
    deviceId = f->deviceId;
    subType = f->subType;
    data.resize(sizeof(MStreamFragmentHeader));
    MStreamFragmentHeader *h = (MStreamFragmentHeader *)data.data();
    std::memcpy(h, f, sizeof(MStreamFragmentHeader));
}

void BaseFragment::insert(const MStreamFragmentHeader *f)
{
    const int start = f->getFragOffset(bigFragId);
    const int end = start + f->fragmentLength;
    if (data.size() < end+(int)sizeof(MStreamFragmentHeader))
        data.resize(end+sizeof(MStreamFragmentHeader));
    parts[start] = end;
    std::memcpy(data.data()+start+sizeof(MStreamFragmentHeader),
                (const char *)f + sizeof(MStreamFragmentHeader),
                f->fragmentLength);
}
